@@ -7,39 +7,23 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- *  The contents of this file are subject to the Mozilla Public License
- *  Version 1.0 (the "License"); you may not use this file except in
- *  compliance with the License. You may obtain a copy of the License at
- *  http://www.mozilla.org/MPL/
+ * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
- *  Software distributed under the License is distributed on an "AS IS"
- *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
- *  License for the specific language governing rights and limitations
- *  under the License.
+ * In order to be useful for every potential user, curl and libcurl are
+ * dual-licensed under the MPL and the MIT/X-derivate licenses.
  *
- *  The Original Code is Curl.
+ * You may opt to use, copy, modify, merge, publish, distribute and/or sell
+ * copies of the Software, and permit persons to whom the Software is
+ * furnished to do so, under the terms of the MPL or the MIT/X-derivate
+ * licenses. You may pick one of these licenses.
  *
- *  The Initial Developer of the Original Code is Daniel Stenberg.
+ * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+ * KIND, either express or implied.
  *
- *  Portions created by the Initial Developer are Copyright (C) 1998.
- *  All Rights Reserved.
- *
- * ------------------------------------------------------------
- * Main author:
- * - Daniel Stenberg <Daniel.Stenberg@haxx.nu>
- *
- * 	http://curl.haxx.nu
- *
- * $Source: /cvsroot/curl/include/curl/curl.h,v $
- * $Revision: 1.35 $
- * $Date: 2000/11/28 09:10:04 $
- * $Author: bagder $
- * $State: Exp $
- * $Locker:  $
- *
- * ------------------------------------------------------------
- ****************************************************************************/
+ * $Id: curl.h,v 1.52 2001/02/07 09:49:06 bagder Exp $
+ *****************************************************************************/
 
+#include <stdio.h>
 /* The include stuff here is mainly for time_t! */
 #ifdef vms
 # include <types.h>
@@ -182,10 +166,6 @@ typedef enum {
 
 #define CURL_ERROR_SIZE 256
 
-/* maximum URL length we deal with in headers */
-#define URL_MAX_LENGTH 4096 
-#define URL_MAX_LENGTH_TXT "4095"
-
 /* name is uppercase CURLOPT_<name>,
    type is one of the defined CURLOPTTYPE_<type>
    number is unique identifier */
@@ -451,8 +431,10 @@ typedef enum {
 
    NOTE: they return TRUE if the strings match *case insensitively*.
  */
-extern int (strequal)(const char *s1, const char *s2);
-extern int (strnequal)(const char *s1, const char *s2, size_t n);
+extern int (Curl_strequal)(const char *s1, const char *s2);
+extern int (Curl_strnequal)(const char *s1, const char *s2, size_t n);
+#define strequal(a,b) Curl_strequal(a,b)
+#define strnequal(a,b,c) Curl_strnequal(a,b,c)
 
 /* external form function */
 int curl_formparse(char *string,
@@ -470,8 +452,8 @@ char *curl_getenv(char *variable);
 char *curl_version(void);
 
 /* This is the version number */
-#define LIBCURL_VERSION "7.5"
-#define LIBCURL_VERSION_NUM 0x070500
+#define LIBCURL_VERSION "7.6.1"
+#define LIBCURL_VERSION_NUM 0x070601
 
 /* linked-list structure for the CURLOPT_QUOTE option (and other) */
 struct curl_slist {
@@ -572,7 +554,7 @@ CURLcode curl_setopt(CURL *handle, CURLoption option, ...);
 CURLcode curl_close(CURL *curl); /* the opposite of curl_open() */
 
 CURLcode curl_read(CURLconnect *c_conn, char *buf, size_t buffersize,
-                   size_t *n);
+                   ssize_t *n);
 CURLcode curl_write(CURLconnect *c_conn, char *buf, size_t amount,
                     size_t *n);
 
@@ -694,7 +676,10 @@ typedef enum {
   CURLINFO_SSL_VERIFYRESULT = CURLINFO_LONG   + 13,
   CURLINFO_FILETIME         = CURLINFO_LONG   + 14,
 
-  CURLINFO_LASTONE          = 15
+  CURLINFO_CONTENT_LENGTH_DOWNLOAD   = CURLINFO_DOUBLE + 15,
+  CURLINFO_CONTENT_LENGTH_UPLOAD     = CURLINFO_DOUBLE + 16,
+
+  CURLINFO_LASTONE          = 17
 } CURLINFO;
 
 /*
