@@ -5,44 +5,28 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- *  The contents of this file are subject to the Mozilla Public License
- *  Version 1.0 (the "License"); you may not use this file except in
- *  compliance with the License. You may obtain a copy of the License at
- *  http://www.mozilla.org/MPL/
+ * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
- *  Software distributed under the License is distributed on an "AS IS"
- *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
- *  License for the specific language governing rights and limitations
- *  under the License.
+ * In order to be useful for every potential user, curl and libcurl are
+ * dual-licensed under the MPL and the MIT/X-derivate licenses.
  *
- *  The Original Code is Curl.
+ * You may opt to use, copy, modify, merge, publish, distribute and/or sell
+ * copies of the Software, and permit persons to whom the Software is
+ * furnished to do so, under the terms of the MPL or the MIT/X-derivate
+ * licenses. You may pick one of these licenses.
  *
- *  The Initial Developer of the Original Code is Daniel Stenberg.
+ * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+ * KIND, either express or implied.
  *
- *  Portions created by the Initial Developer are Copyright (C) 1998.
- *  All Rights Reserved.
- *
- * ------------------------------------------------------------
- * Main author:
- * - Daniel Stenberg <daniel@haxx.se>
- *
- * 	http://curl.haxx.se
- *
- * $Source: /cvsroot/curl/lib/ftp.c,v $
- * $Revision: 1.36 $
- * $Date: 2000/11/30 21:59:51 $
- * $Author: bagder $
- * $State: Exp $
- * $Locker:  $
- *
- * ------------------------------------------------------------
- ****************************************************************************/
+ * $Id: ftp.c,v 1.52 2001/02/13 13:34:16 bagder Exp $
+ *****************************************************************************/
 
 #include "setup.h"
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <ctype.h>
 #include <errno.h>
 
@@ -83,18 +67,27 @@
 #include "if2ip.h"
 #include "hostip.h"
 #include "progress.h"
-#include "download.h"
+#include "transfer.h"
 #include "escape.h"
 #include "http.h" /* for HTTP proxy tunnel stuff */
+#include "ftp.h"
 
 #ifdef KRB4
 #include "security.h"
+#include "krb4.h"
 #endif
+
+#define _MPRINTF_REPLACE /* use our functions only */
+#include <curl/mprintf.h>
+
 /* The last #include file should be: */
 #ifdef MALLOCDEBUG
 #include "memdebug.h"
 #endif
 
+/* easy-to-use macro: */
+#define ftpsendf Curl_ftpsendf
+
 /* returns last node in linked list */
 static struct curl_slist *slist_get_last(struct curl_slist *list)
 {
@@ -197,6 +190,8 @@ static CURLcode AllowServerConnect(struct UrlData *data,
       getsockname(sock, (struct sockaddr *) &add, (int *)&size);
       s=accept(sock, (struct sockaddr *) &add, (int *)&size);
 
+      sclose(sock); /* close the first socket */
+
       if( -1 == s) {
 	/* DIE! */
 	failf(data, "Error accept()ing server connect");
@@ -217,12 +212,16 @@ static CURLcode AllowServerConnect(struct UrlData *data,
 #define lastline(line) (isdigit((int)line[0]) && isdigit((int)line[1]) && \
 			isdigit((int)line[2]) && (' ' == line[3]))
 
-int GetLastResponse(int sockfd, char *buf,
-                    struct connectdata *conn,
-                    int *ftpcode)
+/*
+ * We allow the ftpcode pointer to be NULL if no reply integer is wanted
+ */
+
+int Curl_GetFTPResponse(int sockfd, char *buf,
+                        struct connectdata *conn,
+                        int *ftpcode)
 {
   int nread;
-  int keepon=TRUE;
+  ssize_t keepon=TRUE;
   char *ptr;
   int timeout = 3600; /* in seconds */
   struct timeval interval;
@@ -235,12 +234,13 @@ int GetLastResponse(int sockfd, char *buf,
 #define SELECT_TIMEOUT 2
   int error = SELECT_OK;
 
-  *ftpcode=0; /* 0 for errors */
+  if(ftpcode)
+    *ftpcode=0; /* 0 for errors */
 
   if(data->timeout) {
     /* if timeout is requested, find out how much remaining time we have */
     timeout = data->timeout - /* timeout time */
-      (tvlong(tvnow()) - tvlong(conn->now)); /* spent time */
+      (Curl_tvlong(Curl_tvnow()) - Curl_tvlong(conn->now)); /* spent time */
     if(timeout <=0 ) {
       failf(data, "Transfer aborted due to timeout");
       return -SELECT_TIMEOUT; /* already too little time */
@@ -272,22 +272,21 @@ int GetLastResponse(int sockfd, char *buf,
         break;
       case 0: /* timeout */
         error = SELECT_TIMEOUT;
-        infof(data, "Transfer aborted due to timeout\n");
         failf(data, "Transfer aborted due to timeout");
         break;
       default:
-#ifdef USE_SSLEAY
-        if (data->ssl.use) {
-          keepon = SSL_read(data->ssl.handle, ptr, 1);
+        /*
+         * This code previously didn't use the kerberos sec_read() code
+         * to read, but when we use Curl_read() it may do so. Do confirm
+         * that this is still ok and then remove this comment!
+         */
+        if(CURLE_OK != Curl_read(conn, sockfd, ptr, 1, &keepon))
+          keepon = FALSE;
+        else if(keepon <= 0) {
+          error = SELECT_ERROR;
+          failf(data, "Connection aborted");
         }
-        else {
-#endif
-          keepon = sread(sockfd, ptr, 1);
-#ifdef USE_SSLEAY
-        }
-#endif /* USE_SSLEAY */
-
-        if ((*ptr == '\n') || (*ptr == '\r'))
+        else if ((*ptr == '\n') || (*ptr == '\r'))
           keepon = FALSE;
       }
       if(keepon) {
@@ -321,13 +320,14 @@ int GetLastResponse(int sockfd, char *buf,
   if(error)
     return -error;
 
-  *ftpcode=atoi(buf); /* return the initial number like this */
+  if(ftpcode)
+    *ftpcode=atoi(buf); /* return the initial number like this */
 
   return nread;
 }
 
 /* -- who are we? -- */
-char *getmyhost(char *buf, int buf_size)
+char *Curl_getmyhost(char *buf, int buf_size)
 {
 #if defined(HAVE_GETHOSTNAME)
   gethostname(buf, buf_size);
@@ -345,7 +345,7 @@ char *getmyhost(char *buf, int buf_size)
 
 /* ftp_connect() should do everything that is to be considered a part
    of the connection phase. */
-CURLcode ftp_connect(struct connectdata *conn)
+CURLcode Curl_ftp_connect(struct connectdata *conn)
 {
   /* this is FTP and no proxy */
   int nread;
@@ -371,14 +371,14 @@ CURLcode ftp_connect(struct connectdata *conn)
 
   if (data->bits.tunnel_thru_httpproxy) {
     /* We want "seamless" FTP operations through HTTP proxy tunnel */
-    result = GetHTTPProxyTunnel(data, data->firstsocket,
-                                data->hostname, data->remote_port);
+    result = Curl_ConnectHTTPProxyTunnel(conn, data->firstsocket,
+                                         data->hostname, data->remote_port);
     if(CURLE_OK != result)
       return result;
   }
 
   /* The first thing we do is wait for the "220*" line: */
-  nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+  nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
   if(nread < 0)
     return CURLE_OPERATION_TIMEOUTED;
 
@@ -411,7 +411,7 @@ CURLcode ftp_connect(struct connectdata *conn)
   ftpsendf(data->firstsocket, conn, "USER %s", ftp->user);
 
   /* wait for feedback */
-  nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+  nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
   if(nread < 0)
     return CURLE_OPERATION_TIMEOUTED;
 
@@ -425,7 +425,7 @@ CURLcode ftp_connect(struct connectdata *conn)
     /* 331 Password required for ...
        (the server requires to send the user's password too) */
     ftpsendf(data->firstsocket, conn, "PASS %s", ftp->passwd);
-    nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -474,7 +474,7 @@ CURLcode ftp_connect(struct connectdata *conn)
 
 
 /* argument is already checked for validity */
-CURLcode ftp_done(struct connectdata *conn)
+CURLcode Curl_ftp_done(struct connectdata *conn)
 {
   struct UrlData *data = conn->data;
   struct FTP *ftp = data->proto.ftp;
@@ -511,7 +511,7 @@ CURLcode ftp_done(struct connectdata *conn)
   if(!data->bits.no_body) {  
     /* now let's see what the server says about the transfer we
        just performed: */
-    nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -531,11 +531,11 @@ CURLcode ftp_done(struct connectdata *conn)
       if (qitem->data) {
         ftpsendf(data->firstsocket, conn, "%s", qitem->data);
 
-        nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+        nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
         if(nread < 0)
           return CURLE_OPERATION_TIMEOUTED;
 
-        if (buf[0] != '2') {
+        if (ftpcode >= 400) {
           failf(data, "QUOT string not accepted: %s",
                 qitem->data);
           return CURLE_FTP_QUOTE_ERROR;
@@ -545,9 +545,6 @@ CURLcode ftp_done(struct connectdata *conn)
     }
   }
 
-  free(ftp);
-  data->proto.ftp=NULL; /* it is gone */
-
   return CURLE_OK;
 }
 
@@ -568,6 +565,9 @@ CURLcode _ftp(struct connectdata *conn)
 #if defined (HAVE_INET_NTOA_R)
   char ntoa_buf[64];
 #endif
+#ifdef ENABLE_IPV6
+  struct addrinfo *ai;
+#endif
 
   struct curl_slist *qitem; /* QUOTE item */
   /* the ftp struct is already inited in ftp_connect() */
@@ -585,11 +585,11 @@ CURLcode _ftp(struct connectdata *conn)
       if (qitem->data) {
         ftpsendf(data->firstsocket, conn, "%s", qitem->data);
 
-        nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+        nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
         if(nread < 0)
           return CURLE_OPERATION_TIMEOUTED;
 
-        if (buf[0] != '2') {
+        if (ftpcode >= 400) {
           failf(data, "QUOT string not accepted: %s",
                 qitem->data);
           return CURLE_FTP_QUOTE_ERROR;
@@ -602,7 +602,7 @@ CURLcode _ftp(struct connectdata *conn)
   /* change directory first! */
   if(ftp->dir && ftp->dir[0]) {
     ftpsendf(data->firstsocket, conn, "CWD %s", ftp->dir);
-    nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -617,7 +617,7 @@ CURLcode _ftp(struct connectdata *conn)
        again a grey area as the MDTM is not kosher RFC959 */
     ftpsendf(data->firstsocket, conn, "MDTM %s", ftp->file);
 
-    nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -654,7 +654,7 @@ CURLcode _ftp(struct connectdata *conn)
     ftpsendf(data->firstsocket, conn, "TYPE %s",
              (data->bits.ftp_ascii)?"A":"I");
 
-    nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -667,7 +667,7 @@ CURLcode _ftp(struct connectdata *conn)
 
     ftpsendf(data->firstsocket, conn, "SIZE %s", ftp->file);
 
-    nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -679,7 +679,7 @@ CURLcode _ftp(struct connectdata *conn)
     filesize = atoi(buf+4);
 
     sprintf(buf, "Content-Length: %d\r\n", filesize);
-    result = client_write(data, CLIENTWRITE_BOTH, buf, 0);
+    result = Curl_client_write(data, CLIENTWRITE_BOTH, buf, 0);
     if(result)
       return result;
 
@@ -695,7 +695,7 @@ CURLcode _ftp(struct connectdata *conn)
       /* format: "Tue, 15 Nov 1994 12:45:26 GMT" */
       strftime(buf, BUFSIZE-1, "Last-Modified: %a, %d %b %Y %H:%M:%S %Z\r\n",
                tm);
-      result = client_write(data, CLIENTWRITE_BOTH, buf, 0);
+      result = Curl_client_write(data, CLIENTWRITE_BOTH, buf, 0);
       if(result)
         return result;
     }
@@ -706,6 +706,178 @@ CURLcode _ftp(struct connectdata *conn)
 
   /* We have chosen to use the PORT command */
   if(data->bits.ftp_use_port) {
+#ifdef ENABLE_IPV6
+    struct addrinfo hints, *res, *ai;
+    struct sockaddr_storage ss;
+    int sslen;
+    char hbuf[NI_MAXHOST];
+    char *localaddr;
+    struct sockaddr *sa=(struct sockaddr *)&ss;
+#ifdef NI_WITHSCOPEID
+    const int niflags = NI_NUMERICHOST | NI_NUMERICSERV | NI_WITHSCOPEID;
+#else
+    const int niflags = NI_NUMERICHOST | NI_NUMERICSERV;
+#endif
+    unsigned char *ap;
+    unsigned char *pp;
+    int alen, plen;
+    char portmsgbuf[4096], tmp[4096];
+    char *p;
+    char *mode[] = { "EPRT", "LPRT", "PORT", NULL };
+    char **modep;
+
+    /*
+     * we should use Curl_if2ip?  given pickiness of recent ftpd,
+     * I believe we should use the same address as the control connection.
+     */
+    sslen = sizeof(ss);
+    if (getsockname(data->firstsocket, (struct sockaddr *)&ss, &sslen) < 0)
+      return CURLE_FTP_PORT_FAILED;
+
+    if (getnameinfo((struct sockaddr *)&ss, sslen, hbuf, sizeof(hbuf), NULL, 0,
+	niflags))
+      return CURLE_FTP_PORT_FAILED;
+
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = sa->sa_family;
+    /*hints.ai_family = ss.ss_family;
+      this way can be used if sockaddr_storage is properly defined, as glibc 
+      2.1.X doesn't do*/
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_flags = AI_PASSIVE;
+    if (getaddrinfo(hbuf, "0", &hints, &res))
+      return CURLE_FTP_PORT_FAILED;
+
+    portsock = -1;
+    for (ai = res; ai; ai = ai->ai_next) {
+      portsock = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
+      if (portsock < 0)
+	continue;
+
+      if (bind(portsock, ai->ai_addr, ai->ai_addrlen) < 0) {
+	close(portsock);
+	portsock = -1;
+	continue;
+      }
+
+      if (listen(portsock, 1) < 0) {
+	close(portsock);
+	portsock = -1;
+	continue;
+      }
+
+      break;
+    }
+    if (portsock < 0) {
+      failf(data, strerror(errno));
+      freeaddrinfo(res);
+      return CURLE_FTP_PORT_FAILED;
+    }
+
+    sslen = sizeof(ss);
+    if (getsockname(portsock, sa, &sslen) < 0) {
+      failf(data, strerror(errno));
+      freeaddrinfo(res);
+      return CURLE_FTP_PORT_FAILED;
+    }
+
+    for (modep = mode; modep && *modep; modep++) {
+      int lprtaf, eprtaf;
+
+      switch (sa->sa_family) {
+      case AF_INET:
+	ap = (char *)&((struct sockaddr_in *)&ss)->sin_addr;
+	alen = sizeof(((struct sockaddr_in *)&ss)->sin_addr);
+	pp = (char *)&((struct sockaddr_in *)&ss)->sin_port;
+	plen = sizeof(((struct sockaddr_in *)&ss)->sin_port);
+	lprtaf = 4;
+	eprtaf = 1;
+	break;
+      case AF_INET6:
+	ap = (char *)&((struct sockaddr_in6 *)&ss)->sin6_addr;
+	alen = sizeof(((struct sockaddr_in6 *)&ss)->sin6_addr);
+	pp = (char *)&((struct sockaddr_in6 *)&ss)->sin6_port;
+	plen = sizeof(((struct sockaddr_in6 *)&ss)->sin6_port);
+	lprtaf = 6;
+	eprtaf = 2;
+	break;
+      default:
+	ap = pp = NULL;
+	lprtaf = eprtaf = -1;
+	break;
+      }
+
+      if (strcmp(*modep, "EPRT") == 0) {
+	if (eprtaf < 0)
+	  continue;
+	if (getnameinfo((struct sockaddr *)&ss, sslen,
+	    portmsgbuf, sizeof(portmsgbuf), tmp, sizeof(tmp), niflags))
+	  continue;
+	/* do not transmit IPv6 scope identifier to the wire */
+	if (sa->sa_family == AF_INET6) {
+	  char *q = strchr(portmsgbuf, '%');
+	  if (q)
+	    *q = '\0';
+	}
+	ftpsendf(data->firstsocket, conn, "%s |%d|%s|%s|", *modep, eprtaf,
+	    portmsgbuf, tmp);
+      } else if (strcmp(*modep, "LPRT") == 0 || strcmp(*modep, "PORT") == 0) {
+	int i;
+
+        if (strcmp(*modep, "LPRT") == 0 && lprtaf < 0)
+	  continue;
+        if (strcmp(*modep, "PORT") == 0 && sa->sa_family != AF_INET)
+	  continue;
+
+	portmsgbuf[0] = '\0';
+        if (strcmp(*modep, "LPRT") == 0) {
+	  snprintf(tmp, sizeof(tmp), "%d,%d", lprtaf, alen);
+	  if (strlcat(portmsgbuf, tmp, sizeof(portmsgbuf)) >= sizeof(portmsgbuf)) {
+	    goto again;
+	  }
+	}
+	for (i = 0; i < alen; i++) {
+	  if (portmsgbuf[0])
+	    snprintf(tmp, sizeof(tmp), ",%u", ap[i]);
+	  else
+	    snprintf(tmp, sizeof(tmp), "%u", ap[i]);
+	  if (strlcat(portmsgbuf, tmp, sizeof(portmsgbuf)) >= sizeof(portmsgbuf)) {
+	    goto again;
+	  }
+	}
+        if (strcmp(*modep, "LPRT") == 0) {
+	  snprintf(tmp, sizeof(tmp), ",%d", plen);
+	  if (strlcat(portmsgbuf, tmp, sizeof(portmsgbuf)) >= sizeof(portmsgbuf))
+	    goto again;
+	}
+	for (i = 0; i < plen; i++) {
+	  snprintf(tmp, sizeof(tmp), ",%u", pp[i]);
+	  if (strlcat(portmsgbuf, tmp, sizeof(portmsgbuf)) >= sizeof(portmsgbuf)) {
+	    goto again;
+	  }
+	}
+	ftpsendf(data->firstsocket, conn, "%s %s", *modep, portmsgbuf);
+      }
+
+      nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+      if (nread < 0)
+	return CURLE_OPERATION_TIMEOUTED;
+
+      if (ftpcode != 200) {
+	failf(data, "Server does not grok %s", *modep);
+	continue;
+      } else
+	      break;
+again:;
+    }
+
+    if (!*modep) {
+      close(portsock);
+      freeaddrinfo(res);
+      return CURLE_FTP_PORT_FAILED;
+    }
+
+#else
     struct sockaddr_in sa;
     struct hostent *h=NULL;
     char *hostdataptr=NULL;
@@ -714,23 +886,31 @@ CURLcode _ftp(struct connectdata *conn)
     char myhost[256] = "";
 
     if(data->ftpport) {
-      if(if2ip(data->ftpport, myhost, sizeof(myhost))) {
-        h = GetHost(data, myhost, &hostdataptr);
+      if(Curl_if2ip(data->ftpport, myhost, sizeof(myhost))) {
+        h = Curl_gethost(data, myhost, &hostdataptr);
       }
       else {
         if(strlen(data->ftpport)>1)
-          h = GetHost(data, data->ftpport, &hostdataptr);
+          h = Curl_gethost(data, data->ftpport, &hostdataptr);
         if(h)
           strcpy(myhost, data->ftpport); /* buffer overflow risk */
       }
     }
     if(! *myhost) {
-      h=GetHost(data, getmyhost(myhost, sizeof(myhost)), &hostdataptr);
+      h=Curl_gethost(data,
+                     Curl_getmyhost(myhost, sizeof(myhost)),
+                     &hostdataptr);
     }
     infof(data, "We connect from %s\n", myhost);
 
     if ( h ) {
       if( (portsock = socket(AF_INET, SOCK_STREAM, 0)) >= 0 ) {
+
+        /* we set the secondary socket variable to this for now, it
+           is only so that the cleanup function will close it in case
+           we fail before the true secondary stuff is made */
+        data->secondarysocket = portsock;
+
         memset((char *)&sa, 0, sizeof(sa));
         memcpy((char *)&sa.sin_addr,
                h->h_addr,
@@ -797,7 +977,7 @@ CURLcode _ftp(struct connectdata *conn)
             porttouse & 255);
     }
 
-    nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -805,26 +985,43 @@ CURLcode _ftp(struct connectdata *conn)
       failf(data, "Server does not grok PORT, try without it!");
       return CURLE_FTP_PORT_FAILED;
     }     
+#endif /* ENABLE_IPV6 */
   }
   else { /* we use the PASV command */
+#if 0
+    char *mode[] = { "EPSV", "LPSV", "PASV", NULL };
+    int results[] = { 229, 228, 227, 0 };
+#else
+    char *mode[] = { "PASV", NULL };
+    int results[] = { 227, 0 };
+#endif
+    int modeoff;
 
-    ftpsendf(data->firstsocket, conn, "PASV");
+    for (modeoff = 0; mode[modeoff]; modeoff++) {
+      ftpsendf(data->firstsocket, conn, mode[modeoff]);
+      nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+      if(nread < 0)
+	return CURLE_OPERATION_TIMEOUTED;
 
-    nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
-    if(nread < 0)
-      return CURLE_OPERATION_TIMEOUTED;
+      if (ftpcode == results[modeoff])
+	break;
+    }
 
-    if(ftpcode != 227) {
+    if (!mode[modeoff]) {
       failf(data, "Odd return code after PASV");
       return CURLE_FTP_WEIRD_PASV_REPLY;
     }
-    else {
+    else if (strcmp(mode[modeoff], "PASV") == 0) {
       int ip[4];
       int port[2];
       unsigned short newport; /* remote port, not necessary the local one */
       unsigned short connectport; /* the local port connect() should use! */
       char newhost[32];
+#ifdef ENABLE_IPV6
+      struct addrinfo *res;
+#else
       struct hostent *he;
+#endif
       char *str=buf,*ip_addr;
       char *hostdataptr=NULL;
 
@@ -859,20 +1056,78 @@ CURLcode _ftp(struct connectdata *conn)
          * proxy again here. We already have the name info for it since the
          * previous lookup.
          */
+#ifdef ENABLE_IPV6
+        res = conn->res;
+#else
         he = conn->hp;
+#endif
         connectport =
           (unsigned short)data->port; /* we connect to the proxy's port */
       }
       else {
         /* normal, direct, ftp connection */
-        he = GetHost(data, newhost, &hostdataptr);
-        if(!he) {
+#ifdef ENABLE_IPV6
+        res = Curl_getaddrinfo(data, newhost, newport);
+        if(!res)
+#else
+        he = Curl_gethost(data, newhost, &hostdataptr);
+        if(!he)
+#endif
+        {
           failf(data, "Can't resolve new host %s", newhost);
           return CURLE_FTP_CANT_GET_HOST;
         }
         connectport = newport; /* we connect to the remote port */
       }
 	
+#ifdef ENABLE_IPV6
+      data->secondarysocket = -1;
+      for (ai = res; ai; ai = ai->ai_next) {
+	/* XXX for now, we can do IPv4 only */
+	if (ai->ai_family != AF_INET)
+	  continue;
+
+	data->secondarysocket = socket(ai->ai_family, ai->ai_socktype,
+	    ai->ai_protocol);
+	if (data->secondarysocket < 0)
+	  continue;
+
+	if(data->bits.verbose) {
+	  char hbuf[NI_MAXHOST];
+	  char nbuf[NI_MAXHOST];
+	  char sbuf[NI_MAXSERV];
+#ifdef NI_WITHSCOPEID
+	  const int niflags = NI_NUMERICHOST | NI_NUMERICSERV | NI_WITHSCOPEID;
+#else
+	  const int niflags = NI_NUMERICHOST | NI_NUMERICSERV;
+#endif
+	  if (getnameinfo(res->ai_addr, res->ai_addrlen, nbuf, sizeof(nbuf),
+	      sbuf, sizeof(sbuf), niflags)) {
+	    snprintf(nbuf, sizeof(nbuf), "?");
+	    snprintf(sbuf, sizeof(sbuf), "?");
+	  }
+	  if (getnameinfo(res->ai_addr, res->ai_addrlen, hbuf, sizeof(hbuf),
+	      NULL, 0, 0)) {
+	    infof(data, "Connecting to %s port %s\n", nbuf, sbuf);
+	  } else {
+	    infof(data, "Connecting to %s (%s) port %s\n", hbuf, nbuf, sbuf);
+	  }
+	}
+
+	if (connect(data->secondarysocket, ai->ai_addr, ai->ai_addrlen) < 0) {
+	  close(data->secondarysocket);
+	  data->secondarysocket = -1;
+	  continue;
+	}
+
+	break;
+      }
+
+      if (data->secondarysocket < 0) {
+	failf(data, strerror(errno));
+        return CURLE_FTP_CANT_RECONNECT;
+      }
+#else
       data->secondarysocket = socket(AF_INET, SOCK_STREAM, 0);
 
       memset((char *) &serv_addr, '\0', sizeof(serv_addr));
@@ -967,14 +1222,17 @@ CURLcode _ftp(struct connectdata *conn)
         }
         return CURLE_FTP_CANT_RECONNECT;
       }
+#endif /*ENABLE_IPV6*/
 
       if (data->bits.tunnel_thru_httpproxy) {
         /* We want "seamless" FTP operations through HTTP proxy tunnel */
-        result = GetHTTPProxyTunnel(data, data->secondarysocket,
-                                    newhost, newport);
+        result = Curl_ConnectHTTPProxyTunnel(conn, data->secondarysocket,
+                                             newhost, newport);
         if(CURLE_OK != result)
           return result;
       }
+    } else {
+      return CURLE_FTP_CANT_RECONNECT;
     }
   }
   /* we have the (new) data connection ready */
@@ -986,7 +1244,7 @@ CURLcode _ftp(struct connectdata *conn)
     ftpsendf(data->firstsocket, conn, "TYPE %s",
           (data->bits.ftp_ascii)?"A":"I");
 
-    nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -1017,7 +1275,7 @@ CURLcode _ftp(struct connectdata *conn)
 
         ftpsendf(data->firstsocket, conn, "SIZE %s", ftp->file);
 
-        nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+        nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
         if(nread < 0)
           return CURLE_OPERATION_TIMEOUTED;
 
@@ -1078,7 +1336,7 @@ CURLcode _ftp(struct connectdata *conn)
     else
       ftpsendf(data->firstsocket, conn, "STOR %s", ftp->file);
 
-    nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -1099,9 +1357,9 @@ CURLcode _ftp(struct connectdata *conn)
     /* When we know we're uploading a specified file, we can get the file
        size prior to the actual upload. */
 
-    pgrsSetUploadSize(data, data->infilesize);
+    Curl_pgrsSetUploadSize(data, data->infilesize);
 
-    result = Transfer(conn, -1, -1, FALSE, NULL, /* no download */
+    result = Curl_Transfer(conn, -1, -1, FALSE, NULL, /* no download */
                       data->secondarysocket, bytecountp);
     if(result)
       return result;
@@ -1158,7 +1416,7 @@ CURLcode _ftp(struct connectdata *conn)
       /* Set type to ASCII */
       ftpsendf(data->firstsocket, conn, "TYPE A");
 	
-      nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+      nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
       if(nread < 0)
         return CURLE_OPERATION_TIMEOUTED;
 	
@@ -1180,7 +1438,7 @@ CURLcode _ftp(struct connectdata *conn)
       ftpsendf(data->firstsocket, conn, "TYPE %s",
                (data->bits.ftp_ascii)?"A":"I");
 
-      nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+      nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
       if(nread < 0)
         return CURLE_OPERATION_TIMEOUTED;
 
@@ -1201,7 +1459,7 @@ CURLcode _ftp(struct connectdata *conn)
 
         ftpsendf(data->firstsocket, conn, "SIZE %s", ftp->file);
 
-        nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+        nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
         if(nread < 0)
           return CURLE_OPERATION_TIMEOUTED;
 
@@ -1245,7 +1503,7 @@ CURLcode _ftp(struct connectdata *conn)
 
         ftpsendf(data->firstsocket, conn, "REST %d", data->resume_from);
 
-        nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+        nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
         if(nread < 0)
           return CURLE_OPERATION_TIMEOUTED;
 
@@ -1258,7 +1516,7 @@ CURLcode _ftp(struct connectdata *conn)
       ftpsendf(data->firstsocket, conn, "RETR %s", ftp->file);
     }
 
-    nread = GetLastResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -1330,7 +1588,7 @@ CURLcode _ftp(struct connectdata *conn)
       infof(data, "Getting file with size: %d\n", size);
 
       /* FTP download: */
-      result=Transfer(conn, data->secondarysocket, size, FALSE,
+      result=Curl_Transfer(conn, data->secondarysocket, size, FALSE,
                       bytecountp,
                       -1, NULL); /* no upload here */
       if(result)
@@ -1350,7 +1608,7 @@ CURLcode _ftp(struct connectdata *conn)
 /* -- deal with the ftp server!  -- */
 
 /* argument is already checked for validity */
-CURLcode ftp(struct connectdata *conn)
+CURLcode Curl_ftp(struct connectdata *conn)
 {
   CURLcode retcode;
 
@@ -1412,3 +1670,39 @@ CURLcode ftp(struct connectdata *conn)
   return retcode;
 }
 
+/*
+ * ftpsendf() sends the formated string as a ftp command to a ftp server
+ *
+ * NOTE: we build the command in a fixed-length buffer, which sets length
+ * restrictions on the command!
+ *
+ */
+size_t Curl_ftpsendf(int fd, struct connectdata *conn, char *fmt, ...)
+{
+  size_t bytes_written;
+  char s[256];
+
+  va_list ap;
+  va_start(ap, fmt);
+  vsnprintf(s, 250, fmt, ap);
+  va_end(ap);
+
+  if(conn->data->bits.verbose)
+    fprintf(conn->data->err, "> %s\n", s);
+
+  strcat(s, "\r\n"); /* append a trailing CRLF */
+
+#ifdef KRB4
+  if(conn->sec_complete && conn->data->cmdchannel) {
+    bytes_written = sec_fprintf(conn, conn->data->cmdchannel, s);
+    fflush(conn->data->cmdchannel);
+  }
+  else
+#endif /* KRB4 */
+    {
+      bytes_written = swrite(fd, s, strlen(s));
+    }
+  return(bytes_written);
+}
+
+
