@@ -5,38 +5,21 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- *  The contents of this file are subject to the Mozilla Public License
- *  Version 1.0 (the "License"); you may not use this file except in
- *  compliance with the License. You may obtain a copy of the License at
- *  http://www.mozilla.org/MPL/
+ * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
- *  Software distributed under the License is distributed on an "AS IS"
- *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
- *  License for the specific language governing rights and limitations
- *  under the License.
+ * In order to be useful for every potential user, curl and libcurl are
+ * dual-licensed under the MPL and the MIT/X-derivate licenses.
  *
- *  The Original Code is Curl.
+ * You may opt to use, copy, modify, merge, publish, distribute and/or sell
+ * copies of the Software, and permit persons to whom the Software is
+ * furnished to do so, under the terms of the MPL or the MIT/X-derivate
+ * licenses. You may pick one of these licenses.
  *
- *  The Initial Developer of the Original Code is Daniel Stenberg.
+ * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+ * KIND, either express or implied.
  *
- *  Portions created by the Initial Developer are Copyright (C) 1998.
- *  All Rights Reserved.
- *
- * ------------------------------------------------------------
- * Main author:
- * - Daniel Stenberg <daniel@haxx.se>
- *
- * 	http://curl.haxx.se
- *
- * $Source: /cvsroot/curl/lib/http.c,v $
- * $Revision: 1.37 $
- * $Date: 2000/11/21 09:31:03 $
- * $Author: bagder $
- * $State: Exp $
- * $Locker:  $
- *
- * ------------------------------------------------------------
- ****************************************************************************/
+ * $Id: http.c,v 1.48 2001/02/13 13:34:16 bagder Exp $
+ *****************************************************************************/
 
 #include "setup.h"
 
@@ -104,14 +87,13 @@
 
 #include "urldata.h"
 #include <curl/curl.h>
-#include "download.h"
+#include "transfer.h"
 #include "sendf.h"
 #include "formdata.h"
 #include "progress.h"
 #include "base64.h"
 #include "cookie.h"
 #include "strequal.h"
-#include "url.h"
 #include "ssluse.h"
 
 #define _MPRINTF_REPLACE /* use our functions only */
@@ -122,6 +104,144 @@
 #include "memdebug.h"
 #endif
 
+/*
+ * The add_buffer series of functions are used to build one large memory chunk
+ * from repeated function invokes. Used so that the entire HTTP request can
+ * be sent in one go.
+ */
+static CURLcode
+ add_buffer(send_buffer *in, void *inptr, size_t size);
+
+/*
+ * add_buffer_init() returns a fine buffer struct
+ */
+static
+send_buffer *add_buffer_init(void)
+{
+  send_buffer *blonk;
+  blonk=(send_buffer *)malloc(sizeof(send_buffer));
+  if(blonk) {
+    memset(blonk, 0, sizeof(send_buffer));
+    return blonk;
+  }
+  return NULL; /* failed, go home */
+}
+
+/*
+ * add_buffer_send() sends a buffer and frees all associated memory.
+ */
+static
+size_t add_buffer_send(int sockfd, struct connectdata *conn, send_buffer *in)
+{
+  size_t amount;
+  if(conn->data->bits.verbose) {
+    fputs("> ", conn->data->err);
+    /* this data _may_ contain binary stuff */
+    fwrite(in->buffer, in->size_used, 1, conn->data->err);
+  }
+
+  Curl_write(conn, sockfd, in->buffer, in->size_used, &amount);
+
+  if(in->buffer)
+    free(in->buffer);
+  free(in);
+
+  return amount;
+}
+
+
+/* 
+ * add_bufferf() builds a buffer from the formatted input
+ */
+static
+CURLcode add_bufferf(send_buffer *in, char *fmt, ...)
+{
+  CURLcode result = CURLE_OUT_OF_MEMORY;
+  char *s;
+  va_list ap;
+  va_start(ap, fmt);
+  s = vaprintf(fmt, ap); /* this allocs a new string to append */
+  va_end(ap);
+
+  if(s) {
+    result = add_buffer(in, s, strlen(s));
+    free(s);
+  }
+  return result;
+}
+
+/*
+ * add_buffer() appends a memory chunk to the existing one
+ */
+static
+CURLcode add_buffer(send_buffer *in, void *inptr, size_t size)
+{
+  char *new_rb;
+  int new_size;
+
+  if(size > 0) {
+    if(!in->buffer ||
+       ((in->size_used + size) > (in->size_max - 1))) {
+      new_size = (in->size_used+size)*2;
+      if(in->buffer)
+        /* we have a buffer, enlarge the existing one */
+        new_rb = (char *)realloc(in->buffer, new_size);
+      else
+        /* create a new buffer */
+        new_rb = (char *)malloc(new_size);
+
+      if(!new_rb)
+        return CURLE_OUT_OF_MEMORY;
+
+      in->buffer = new_rb;
+      in->size_max = new_size;
+    }
+    memcpy(&in->buffer[in->size_used], inptr, size);
+      
+    in->size_used += size;
+  }
+
+  return CURLE_OK;
+}
+
+/* end of the add_buffer functions */
+/*****************************************************************************/
+
+/*
+ * Read everything until a newline.
+ */
+
+static
+int GetLine(int sockfd, char *buf, struct connectdata *conn)
+{
+  ssize_t nread;
+  int read_rc=1;
+  char *ptr;
+  struct UrlData *data=conn->data;
+
+  ptr=buf;
+
+  /* get us a full line, terminated with a newline */
+  for(nread=0;
+      (nread<BUFSIZE) && read_rc;
+      nread++, ptr++) {
+    if((CURLE_OK != Curl_read(conn, sockfd, ptr, 1, &nread)) ||
+       (nread <= 0) ||
+       (*ptr == '\n'))
+      break;
+  }
+  *ptr=0; /* zero terminate */
+  
+  if(data->bits.verbose) {
+    fputs("< ", data->err);
+    fwrite(buf, 1, nread, data->err);
+    fputs("\n", data->err);
+  }
+  return nread>0?nread:0;
+}
+
+
+
 /*
  * This function checks the linked list of custom HTTP headers for a particular
  * header (prefix).
@@ -140,32 +260,34 @@ bool static checkheaders(struct UrlData *data, char *thisheader)
 }
 
 /*
- * GetHTTPProxyTunnel() requires that we're connected to a HTTP proxy. This
+ * ConnectHTTPProxyTunnel() requires that we're connected to a HTTP proxy. This
  * function will issue the necessary commands to get a seamless tunnel through
  * this proxy. After that, the socket can be used just as a normal socket.
  */
 
-CURLcode GetHTTPProxyTunnel(struct UrlData *data, int tunnelsocket,
-                            char *hostname, int remote_port)
+CURLcode Curl_ConnectHTTPProxyTunnel(struct connectdata *conn,
+                                     int tunnelsocket,
+                                     char *hostname, int remote_port)
 {
   int httperror=0;
   int subversion=0;
+  struct UrlData *data=conn->data;
 
   infof(data, "Establish HTTP proxy tunnel to %s:%d\n", hostname, remote_port);
 
-  /* OK, now send the connect statment */
-  sendf(tunnelsocket, data,
-        "CONNECT %s:%d HTTP/1.0\015\012"
-        "%s"
-        "%s"
-        "\r\n",
-        hostname, remote_port,
-        (data->bits.proxy_user_passwd)?data->ptr_proxyuserpwd:"",
-        (data->useragent?data->ptr_uagent:"")
-        );
+  /* OK, now send the connect request to the proxy */
+  Curl_sendf(tunnelsocket, conn,
+             "CONNECT %s:%d HTTP/1.0\015\012"
+             "%s"
+             "%s"
+             "\r\n",
+             hostname, remote_port,
+             (data->bits.proxy_user_passwd)?data->ptr_proxyuserpwd:"",
+             (data->useragent?data->ptr_uagent:"")
+             );
 
   /* wait for the proxy to send us a HTTP/1.0 200 OK header */
-  while(GetLine(tunnelsocket, data->buffer, data)) {
+  while(GetLine(tunnelsocket, data->buffer, conn)) {
     if('\r' == data->buffer[0])
       break; /* end of headers */
     if(2 == sscanf(data->buffer, "HTTP/1.%d %d",
@@ -187,7 +309,7 @@ CURLcode GetHTTPProxyTunnel(struct UrlData *data, int tunnelsocket,
   return CURLE_OK;
 }
 
-CURLcode http_connect(struct connectdata *conn)
+CURLcode Curl_http_connect(struct connectdata *conn)
 {
   struct UrlData *data;
   CURLcode result;
@@ -203,16 +325,15 @@ CURLcode http_connect(struct connectdata *conn)
   if (conn->protocol & PROT_HTTPS) {
     if (data->bits.httpproxy) {
       /* HTTPS through a proxy can only be done with a tunnel */
-      result = GetHTTPProxyTunnel(data, data->firstsocket,
-                                  data->hostname, data->remote_port);
+      result = Curl_ConnectHTTPProxyTunnel(conn, data->firstsocket,
+                                           data->hostname, data->remote_port);
       if(CURLE_OK != result)
         return result;
     }
 
     /* now, perform the SSL initialization for this socket */
-    if(UrgSSLConnect (data)) {
+    if(Curl_SSLConnect(data))
       return CURLE_SSL_CONNECT_ERROR;
-    }
   }
 
   if(data->bits.user_passwd && !data->bits.this_is_a_follow) {
@@ -226,14 +347,14 @@ CURLcode http_connect(struct connectdata *conn)
 
 /* called from curl_close() when this struct is about to get wasted, free
    protocol-specific resources */
-CURLcode http_close(struct connectdata *conn)
+CURLcode Curl_http_close(struct connectdata *conn)
 {
   if(conn->data->auth_host)
     free(conn->data->auth_host);
   return CURLE_OK;
 }
 
-CURLcode http_done(struct connectdata *conn)
+CURLcode Curl_http_done(struct connectdata *conn)
 {
   struct UrlData *data;
   long *bytecount = &conn->bytecount;
@@ -245,7 +366,7 @@ CURLcode http_done(struct connectdata *conn)
   if(data->bits.http_formpost) {
     *bytecount = http->readbytecount + http->writebytecount;
       
-    FormFree(http->sendit); /* Now free that whole lot */
+    Curl_FormFree(http->sendit); /* Now free that whole lot */
 
     data->fread = http->storefread; /* restore */
     data->in = http->in; /* restore */
@@ -254,14 +375,11 @@ CURLcode http_done(struct connectdata *conn)
     *bytecount = http->readbytecount + http->writebytecount;
   }
 
-  free(http);
-  data->proto.http=NULL; /* it is gone */
-
   return CURLE_OK;
 }
 
 
-CURLcode http(struct connectdata *conn)
+CURLcode Curl_http(struct connectdata *conn)
 {
   struct UrlData *data=conn->data;
   char *buf = data->buffer; /* this is a short cut to the buffer */
@@ -301,29 +419,26 @@ CURLcode http(struct connectdata *conn)
        !data->auth_host ||
        strequal(data->auth_host, data->hostname)) {
       sprintf(data->buffer, "%s:%s", data->user, data->passwd);
-      if(base64_encode(data->buffer, strlen(data->buffer),
-                      &authorization) >= 0) {
-        data->ptr_userpwd = maprintf( "Authorization: Basic %s\015\012",
-                                      authorization);
+      if(Curl_base64_encode(data->buffer, strlen(data->buffer),
+                            &authorization) >= 0) {
+        data->ptr_userpwd = aprintf( "Authorization: Basic %s\015\012",
+                                     authorization);
         free(authorization);
       }
     }
   }
-  if((data->bits.set_range) && !checkheaders(data, "Range:")) {
-    data->ptr_rangeline = maprintf("Range: bytes=%s\015\012", data->range);
-  }
   if((data->bits.http_set_referer) && !checkheaders(data, "Referer:")) {
-    data->ptr_ref = maprintf("Referer: %s\015\012", data->referer);
+    data->ptr_ref = aprintf("Referer: %s\015\012", data->referer);
   }
   if(data->cookie && !checkheaders(data, "Cookie:")) {
-    data->ptr_cookie = maprintf("Cookie: %s\015\012", data->cookie);
+    data->ptr_cookie = aprintf("Cookie: %s\015\012", data->cookie);
   }
 
   if(data->cookies) {
-    co = cookie_getlist(data->cookies,
-                        host,
-                        ppath,
-                        conn->protocol&PROT_HTTPS?TRUE:FALSE);
+    co = Curl_cookie_getlist(data->cookies,
+                             host,
+                             ppath,
+                             conn->protocol&PROT_HTTPS?TRUE:FALSE);
   }
   if ((data->bits.httpproxy) && !(conn->protocol&PROT_HTTPS))  {
     /* The path sent to the proxy is in fact the entire URL */
@@ -332,7 +447,7 @@ CURLcode http(struct connectdata *conn)
   if(data->bits.http_formpost) {
     /* we must build the whole darned post sequence first, so that we have
        a size of the whole shebang before we start to send it */
-    http->sendit = getFormData(data->httppost, &http->postsize);
+    http->sendit = Curl_getFormData(data->httppost, &http->postsize);
   }
 
   if(!checkheaders(data, "Host:")) {
@@ -340,9 +455,9 @@ CURLcode http(struct connectdata *conn)
        (!(conn->protocol&PROT_HTTPS) && (data->remote_port == PORT_HTTP)) )
       /* If (HTTPS on port 443) OR (non-HTTPS on port 80) then don't include
          the port number in the host string */
-      data->ptr_host = maprintf("Host: %s\r\n", host);
+      data->ptr_host = aprintf("Host: %s\r\n", host);
     else
-      data->ptr_host = maprintf("Host: %s:%d\r\n", host, data->remote_port);
+      data->ptr_host = aprintf("Host: %s:%d\r\n", host, data->remote_port);
   }
 
   if(!checkheaders(data, "Pragma:"))
@@ -351,6 +466,92 @@ CURLcode http(struct connectdata *conn)
   if(!checkheaders(data, "Accept:"))
     http->p_accept = "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, */*\r\n";
 
+  if((data->bits.http_post ||
+      data->bits.http_formpost ||
+      data->bits.http_put) &&
+     data->resume_from) {
+    /**********************************************************************
+     * Resuming upload in HTTP means that we PUT or POST and that we have
+     * got a resume_from value set. The resume value has already created
+     * a Range: header that will be passed along. We need to "fast forward"
+     * the file the given number of bytes and decrease the assume upload
+     * file size before we continue this venture in the dark lands of HTTP.
+     *********************************************************************/
+   
+    if(data->resume_from < 0 ) {
+      /*
+       * This is meant to get the size of the present remote-file by itself.
+       * We don't support this now. Bail out!
+       */
+       data->resume_from = 0;
+    }
+
+    if(data->resume_from) {
+      /* do we still game? */
+      int passed=0;
+
+      /* Now, let's read off the proper amount of bytes from the
+         input. If we knew it was a proper file we could've just
+         fseek()ed but we only have a stream here */
+      do {
+        int readthisamountnow = (data->resume_from - passed);
+        int actuallyread;
+
+        if(readthisamountnow > BUFSIZE)
+          readthisamountnow = BUFSIZE;
+
+        actuallyread =
+          data->fread(data->buffer, 1, readthisamountnow, data->in);
+
+        passed += actuallyread;
+        if(actuallyread != readthisamountnow) {
+          failf(data, "Could only read %d bytes from the input\n",
+                passed);
+          return CURLE_READ_ERROR;
+        }
+      } while(passed != data->resume_from); /* loop until done */
+
+      /* now, decrease the size of the read */
+      if(data->infilesize>0) {
+        data->infilesize -= data->resume_from;
+
+        if(data->infilesize <= 0) {
+          failf(data, "File already completely uploaded\n");
+          return CURLE_PARTIAL_FILE;
+        }
+      }
+      /* we've passed, proceed as normal */
+    }
+  }
+  if(data->bits.set_range) {
+    /*
+     * A range is selected. We use different headers whether we're downloading
+     * or uploading and we always let customized headers override our internal
+     * ones if any such are specified.
+     */
+    if((data->httpreq == HTTPREQ_GET) &&
+       !checkheaders(data, "Range:")) {
+      data->ptr_rangeline = aprintf("Range: bytes=%s\r\n", data->range);
+    }
+    else if((data->httpreq != HTTPREQ_GET) &&
+            !checkheaders(data, "Content-Range:")) {
+
+      if(data->resume_from) {
+        /* This is because "resume" was selected */
+        long total_expected_size= data->resume_from + data->infilesize;
+        data->ptr_rangeline = aprintf("Content-Range: bytes %s%ld/%ld\r\n",
+                                      data->range, total_expected_size-1,
+                                      total_expected_size);
+      }
+      else {
+        /* Range was selected and then we just pass the incoming range and 
+           append total size */
+        data->ptr_rangeline = aprintf("Content-Range: bytes %s/%d\r\n",
+                                      data->range, data->infilesize);
+      }
+    }
+  }
+
   do {
     send_buffer *req_buffer;
     struct curl_slist *headers=data->headers;
@@ -406,7 +607,7 @@ CURLcode http(struct connectdata *conn)
       if(count) {
         add_buffer(req_buffer, "\r\n", 2);
       }
-      cookie_freelist(store); /* free the cookie list */
+      Curl_cookie_freelist(store); /* free the cookie list */
       co=NULL;
     }
 
@@ -468,7 +669,7 @@ CURLcode http(struct connectdata *conn)
     }
 
     if(data->bits.http_formpost) {
-      if(FormInit(&http->form, http->sendit)) {
+      if(Curl_FormInit(&http->form, http->sendit)) {
         failf(data, "Internal HTTP POST error!\n");
         return CURLE_HTTP_POST_ERROR;
       }
@@ -478,24 +679,24 @@ CURLcode http(struct connectdata *conn)
           
       data->fread =
         (size_t (*)(char *, size_t, size_t, FILE *))
-        FormReader; /* set the read function to read from the
-                       generated form data */
+        Curl_FormReader; /* set the read function to read from the
+                            generated form data */
       data->in = (FILE *)&http->form;
 
       add_bufferf(req_buffer,
                   "Content-Length: %d\r\n", http->postsize-2);
 
       /* set upload size to the progress meter */
-      pgrsSetUploadSize(data, http->postsize);
+      Curl_pgrsSetUploadSize(data, http->postsize);
 
       data->request_size = 
         add_buffer_send(data->firstsocket, conn, req_buffer);
-      result = Transfer(conn, data->firstsocket, -1, TRUE,
+      result = Curl_Transfer(conn, data->firstsocket, -1, TRUE,
                         &http->readbytecount,
                           data->firstsocket,
                         &http->writebytecount);
       if(result) {
-        FormFree(http->sendit); /* free that whole lot */
+        Curl_FormFree(http->sendit); /* free that whole lot */
         return result;
       }
     }
@@ -511,14 +712,14 @@ CURLcode http(struct connectdata *conn)
         add_bufferf(req_buffer, "\015\012");
 
       /* set the upload size to the progress meter */
-      pgrsSetUploadSize(data, data->infilesize);
+      Curl_pgrsSetUploadSize(data, data->infilesize);
 
       /* this sends the buffer and frees all the buffer resources */
       data->request_size = 
         add_buffer_send(data->firstsocket, conn, req_buffer);
 
       /* prepare for transfer */
-      result = Transfer(conn, data->firstsocket, -1, TRUE,
+      result = Curl_Transfer(conn, data->firstsocket, -1, TRUE,
                         &http->readbytecount,
                         data->firstsocket,
                         &http->writebytecount);
@@ -564,7 +765,7 @@ CURLcode http(struct connectdata *conn)
         add_buffer_send(data->firstsocket, conn, req_buffer);
 
       /* HTTP GET/HEAD download: */
-      result = Transfer(conn, data->firstsocket, -1, TRUE, bytecount,
+      result = Curl_Transfer(conn, data->firstsocket, -1, TRUE, bytecount,
                         -1, NULL); /* nothing to upload */
     }
     if(result)
