@@ -5,38 +5,21 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- *  The contents of this file are subject to the Mozilla Public License
- *  Version 1.0 (the "License"); you may not use this file except in
- *  compliance with the License. You may obtain a copy of the License at
- *  http://www.mozilla.org/MPL/
+ * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
- *  Software distributed under the License is distributed on an "AS IS"
- *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
- *  License for the specific language governing rights and limitations
- *  under the License.
+ * In order to be useful for every potential user, curl and libcurl are
+ * dual-licensed under the MPL and the MIT/X-derivate licenses.
  *
- *  The Original Code is Curl.
+ * You may opt to use, copy, modify, merge, publish, distribute and/or sell
+ * copies of the Software, and permit persons to whom the Software is
+ * furnished to do so, under the terms of the MPL or the MIT/X-derivate
+ * licenses. You may pick one of these licenses.
  *
- *  The Initial Developer of the Original Code is Daniel Stenberg.
+ * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+ * KIND, either express or implied.
  *
- *  Portions created by the Initial Developer are Copyright (C) 1998.
- *  All Rights Reserved.
- *
- * ------------------------------------------------------------
- * Main author:
- * - Daniel Stenberg <daniel@haxx.se>
- *
- * 	http://curl.haxx.se
- *
- * $Source: /cvsroot/curl/lib/progress.c,v $
- * $Revision: 1.18 $
- * $Date: 2000/11/16 07:20:12 $
- * $Author: bagder $
- * $State: Exp $
- * $Locker:  $
- *
- * ------------------------------------------------------------
- ****************************************************************************/
+ * $Id: progress.c,v 1.23 2001/01/27 18:23:59 bagder Exp $
+ *****************************************************************************/
 
 #include "setup.h"
 
@@ -62,7 +45,7 @@
 
 #include "progress.h"
 
-void time2str(char *r, int t)
+static void time2str(char *r, int t)
 {
   int h = (t/3600);
   int m = (t-(h*3600))/60;
@@ -72,7 +55,7 @@ void time2str(char *r, int t)
 
 /* The point of this function would be to return a string of the input data,
    but never longer than 5 columns. Add suffix k, M, G when suitable... */
-char *max5data(double bytes, char *max5)
+static char *max5data(double bytes, char *max5)
 {
 #define ONE_KILOBYTE 1024
 #define ONE_MEGABYTE (1024*1024)
@@ -108,16 +91,16 @@ char *max5data(double bytes, char *max5)
 
 */
 
-void pgrsDone(struct UrlData *data)
+void Curl_pgrsDone(struct UrlData *data)
 {
   if(!(data->progress.flags & PGRS_HIDE)) {
     data->progress.lastshow=0;
-    pgrsUpdate(data); /* the final (forced) update */
+    Curl_pgrsUpdate(data); /* the final (forced) update */
     fprintf(data->err, "\n");
   }
 }
 
-void pgrsTime(struct UrlData *data, timerid timer)
+void Curl_pgrsTime(struct UrlData *data, timerid timer)
 {
   switch(timer) {
   default:
@@ -128,19 +111,19 @@ void pgrsTime(struct UrlData *data, timerid timer)
     /* This is set at the start of a single fetch, there may be several
        fetches within an operation, why we add all other times relative
        to this one */
-    data->progress.t_startsingle = tvnow();
+    data->progress.t_startsingle = Curl_tvnow();
     break;
 
   case TIMER_NAMELOOKUP:
-    data->progress.t_nslookup += tvdiff(tvnow(),
+    data->progress.t_nslookup += Curl_tvdiff(Curl_tvnow(),
                                         data->progress.t_startsingle);
     break;
   case TIMER_CONNECT:
-    data->progress.t_connect += tvdiff(tvnow(),
+    data->progress.t_connect += Curl_tvdiff(Curl_tvnow(),
                                        data->progress.t_startsingle);
     break;
   case TIMER_PRETRANSFER:
-    data->progress.t_pretransfer += tvdiff(tvnow(),
+    data->progress.t_pretransfer += Curl_tvdiff(Curl_tvnow(),
                                            data->progress.t_startsingle);
     break;
   case TIMER_POSTRANSFER:
@@ -149,22 +132,22 @@ void pgrsTime(struct UrlData *data, timerid timer)
   }
 }
 
-void pgrsStartNow(struct UrlData *data)
+void Curl_pgrsStartNow(struct UrlData *data)
 {
-  data->progress.start = tvnow();
+  data->progress.start = Curl_tvnow();
 }
 
-void pgrsSetDownloadCounter(struct UrlData *data, double size)
+void Curl_pgrsSetDownloadCounter(struct UrlData *data, double size)
 {
   data->progress.downloaded = size;
 }
 
-void pgrsSetUploadCounter(struct UrlData *data, double size)
+void Curl_pgrsSetUploadCounter(struct UrlData *data, double size)
 {
   data->progress.uploaded = size;
 }
 
-void pgrsSetDownloadSize(struct UrlData *data, double size)
+void Curl_pgrsSetDownloadSize(struct UrlData *data, double size)
 {
   if(size > 0) {
     data->progress.size_dl = size;
@@ -172,7 +155,7 @@ void pgrsSetDownloadSize(struct UrlData *data, double size)
   }
 }
 
-void pgrsSetUploadSize(struct UrlData *data, double size)
+void Curl_pgrsSetUploadSize(struct UrlData *data, double size)
 {
   if(size > 0) {
     data->progress.size_ul = size;
@@ -188,7 +171,7 @@ void pgrsSetUploadSize(struct UrlData *data, double size)
 
  */
 
-int pgrsUpdate(struct UrlData *data)
+int Curl_pgrsUpdate(struct UrlData *data)
 {
   struct timeval now;
   int result;
@@ -220,6 +203,9 @@ int pgrsUpdate(struct UrlData *data)
          even when not displayed! */
   else if(!(data->progress.flags & PGRS_HEADERS_OUT)) {
     if (!data->progress.callback) {
+      if(data->resume_from)
+        fprintf(data->err, "** Resuming transfer from byte position %d\n",
+                data->resume_from);
       fprintf(data->err,
               "  %% Total    %% Received %% Xferd  Average Speed          Time             Curr.\n"
               "                                 Dload  Upload Total    Current  Left    Speed\n");
@@ -227,16 +213,16 @@ int pgrsUpdate(struct UrlData *data)
     data->progress.flags |= PGRS_HEADERS_OUT; /* headers are shown */
   }
 
-  now = tvnow(); /* what time is it */
+  now = Curl_tvnow(); /* what time is it */
 
-  if(data->progress.lastshow == tvlong(now))
+  /* The exact time spent so far */
+  data->progress.timespent = Curl_tvdiff (now, data->progress.start);
+
+  if(data->progress.lastshow == Curl_tvlong(now))
     return 0; /* never update this more than once a second if the end isn't 
                  reached */
   data->progress.lastshow = now.tv_sec;
 
-  /* The exact time spent so far */
-  data->progress.timespent = tvdiff (now, data->progress.start);
-
   /* The average download speed this far */
   data->progress.dlspeed = data->progress.downloaded/(data->progress.timespent!=0.0?data->progress.timespent:1.0);
 
@@ -274,17 +260,13 @@ int pgrsUpdate(struct UrlData *data)
   }
 
       /* Figure out the estimated time of arrival for the upload */
-  if(data->progress.flags & PGRS_UL_SIZE_KNOWN) {
-    if(!data->progress.ulspeed)
-      data->progress.ulspeed=1;
+  if((data->progress.flags & PGRS_UL_SIZE_KNOWN) && data->progress.ulspeed){
     ulestimate = data->progress.size_ul / data->progress.ulspeed;
     ulpercen = (data->progress.uploaded / data->progress.size_ul)*100;
   }
 
   /* ... and the download */
-  if(data->progress.flags & PGRS_DL_SIZE_KNOWN) {
-    if(!data->progress.dlspeed)
-      data->progress.dlspeed=1;
+  if((data->progress.flags & PGRS_DL_SIZE_KNOWN) && data->progress.dlspeed) {
     dlestimate = data->progress.size_dl / data->progress.dlspeed;
     dlpercen = (data->progress.downloaded / data->progress.size_dl)*100;
   }
