@@ -5,46 +5,21 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- *  The contents of this file are subject to the Mozilla Public License
- *  Version 1.0 (the "License"); you may not use this file except in
- *  compliance with the License. You may obtain a copy of the License at
- *  http://www.mozilla.org/MPL/
+ * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
- *  Software distributed under the License is distributed on an "AS IS"
- *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
- *  License for the specific language governing rights and limitations
- *  under the License.
+ * In order to be useful for every potential user, curl and libcurl are
+ * dual-licensed under the MPL and the MIT/X-derivate licenses.
  *
- *  The Original Code is Curl.
+ * You may opt to use, copy, modify, merge, publish, distribute and/or sell
+ * copies of the Software, and permit persons to whom the Software is
+ * furnished to do so, under the terms of the MPL or the MIT/X-derivate
+ * licenses. You may pick one of these licenses.
  *
- *  The Initial Developer of the Original Code is Daniel Stenberg.
+ * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+ * KIND, either express or implied.
  *
- *  Portions created by the Initial Developer are Copyright (C) 1998.
- *  All Rights Reserved.
- *
- * ------------------------------------------------------------
- * Main author:
- * - Daniel Stenberg <daniel@haxx.se>
- *
- * 	http://curl.haxx.se
- *
- * $Source: /cvsroot/curl/lib/telnet.c,v $
- * $Revision: 1.7 $
- * $Date: 2000/11/22 12:56:27 $
- * $Author: bagder $
- * $State: Exp $
- * $Locker:  $
- *
- * ------------------------------------------------------------
- *
- * This implementation of the TELNET protocol is written by
- * Linus Nielsen <Linus.Nielsen@haxx.se>,
- * with some code snippets stolen from the BSD Telnet client.
- *
- * The negotiation is performed according to RFC 1143 (D. Bernstein,
- * "The Q Method of Implementing TELNET Option Negotiation")
- *
- ****************************************************************************/
+ * $Id: telnet.c,v 1.13 2001/01/31 15:05:44 bagder Exp $
+ *****************************************************************************/
 
 #include "setup.h"
 
@@ -96,17 +71,14 @@
 
 #include "urldata.h"
 #include <curl/curl.h>
-#include "download.h"
+#include "transfer.h"
 #include "sendf.h"
-#include "formdata.h"
-#include "progress.h"
 
 #define _MPRINTF_REPLACE /* use our functions only */
 #include <curl/mprintf.h>
 
 #define  TELOPTS
 #define  TELCMDS
-#define  SLC_NAMES
 
 #include "arpa_telnet.h"
 
@@ -123,10 +95,7 @@
 #define  SB_EOF() (subpointer >= subend)
 #define  SB_LEN() (subend - subpointer)
 
-void telwrite(struct UrlData *data,
-	      unsigned char *buffer,	/* Data to write */
-	      int count);		/* Number of bytes to write */
-
+static
 void telrcv(struct UrlData *data,
 	    unsigned char *inbuf,	/* Data received from socket */
 	    int count);			/* Number of bytes received */
@@ -180,6 +149,7 @@ static int him[256];
 static int himq[256]; 
 static int him_preferred[256]; 
 
+static
 void init_telnet(struct UrlData *data)
 {
    telrcv_state = TS_DATA;
@@ -271,6 +241,7 @@ static void send_negotiation(struct UrlData *data, int cmd, int option)
    printoption(data, "SENT", cmd, option);
 }
 
+static
 void set_remote_option(struct UrlData *data, int option, int newstate)
 {
    if(newstate == YES)
@@ -351,6 +322,7 @@ void set_remote_option(struct UrlData *data, int option, int newstate)
    }
 }
 
+static
 void rec_will(struct UrlData *data, int option)
 {
    switch(him[option])
@@ -402,6 +374,7 @@ void rec_will(struct UrlData *data, int option)
    }
 }
    
+static
 void rec_wont(struct UrlData *data, int option)
 {
    switch(him[option])
@@ -525,6 +498,7 @@ void set_local_option(struct UrlData *data, int option, int newstate)
    }
 }
 
+static
 void rec_do(struct UrlData *data, int option)
 {
    switch(us[option])
@@ -575,7 +549,8 @@ void rec_do(struct UrlData *data, int option)
       break;
    }
 }
-   
+
+static   
 void rec_dont(struct UrlData *data, int option)
 {
    switch(us[option])
@@ -694,6 +669,7 @@ static void suboption(struct UrlData *data)
    return;
 }
 
+static
 void telrcv(struct UrlData *data,
 	    unsigned char *inbuf,	/* Data received from socket */
 	    int count)			/* Number of bytes received */
@@ -714,7 +690,7 @@ void telrcv(struct UrlData *data,
 	    break;   /* Ignore \0 after CR */
 	 }
 
-	 client_write(data, CLIENTWRITE_BODY, (char *)&c, 1);
+	 Curl_client_write(data, CLIENTWRITE_BODY, (char *)&c, 1);
 	 continue;
 
       case TS_DATA:
@@ -728,7 +704,7 @@ void telrcv(struct UrlData *data,
 	    telrcv_state = TS_CR;
 	 }
 
-	 client_write(data, CLIENTWRITE_BODY, (char *)&c, 1);
+	 Curl_client_write(data, CLIENTWRITE_BODY, (char *)&c, 1);
 	 continue;
 
       case TS_IAC:
@@ -752,7 +728,7 @@ void telrcv(struct UrlData *data,
 	   telrcv_state = TS_SB;
 	   continue;
 	case IAC:
-          client_write(data, CLIENTWRITE_BODY, (char *)&c, 1);
+          Curl_client_write(data, CLIENTWRITE_BODY, (char *)&c, 1);
           break;
 	case DM:
 	case NOP:
@@ -843,41 +819,12 @@ void telrcv(struct UrlData *data,
    }
 }
 
-void telwrite(struct UrlData *data,
-	      unsigned char *buffer,	/* Data to write */
-	      int count)		/* Number of bytes to write */
-{
-   unsigned char outbuf[2];
-   int out_count = 0;
-   int bytes_written;
-
-   while(count--)
-   {
-      outbuf[0] = *buffer++;
-      out_count = 1;
-      if(outbuf[0] == IAC)
-	 outbuf[out_count++] = IAC;
-      
-#ifndef USE_SSLEAY
-      bytes_written = swrite(data->firstsocket, outbuf, out_count);
-#else
-      if (data->ssl.use) {
-        bytes_written = SSL_write(data->ssl.handle, (char *)outbuf,
-                                  out_count);
-      }
-      else {
-        bytes_written = swrite(data->firstsocket, outbuf, out_count);
-      }
-#endif /* USE_SSLEAY */
-   }
-}
-
-CURLcode telnet_done(struct connectdata *conn)
+CURLcode Curl_telnet_done(struct connectdata *conn)
 {
   return CURLE_OK;
 }
 
-CURLcode telnet(struct connectdata *conn)
+CURLcode Curl_telnet(struct connectdata *conn)
 {
   struct UrlData *data = conn->data;
   int sockfd = data->firstsocket;
@@ -886,7 +833,7 @@ CURLcode telnet(struct connectdata *conn)
 
   bool keepon = TRUE;
   char *buf = data->buffer;
-  int nread;
+  ssize_t nread;
 
   init_telnet(data);
    
@@ -896,49 +843,51 @@ CURLcode telnet(struct connectdata *conn)
 
   keepfd = readfd;
 
-   while (keepon)
-   {
-      readfd = keepfd;		/* set this every lap in the loop */
+  while (keepon) {
+    readfd = keepfd;		/* set this every lap in the loop */
 
-      switch (select (sockfd + 1, &readfd, NULL, NULL, NULL))
-      {
-      case -1:			/* error, stop reading */
-	 keepon = FALSE;
-	 continue;
-      case 0:			/* timeout */
-	 break;
-      default:			/* read! */
-	 if(FD_ISSET(1, &readfd))
-	 {
-	    nread = read(1, buf, 255);
-	    telwrite(data, (unsigned char *)buf, nread);
-	 }
+    switch (select (sockfd + 1, &readfd, NULL, NULL, NULL)) {
+    case -1:			/* error, stop reading */
+      keepon = FALSE;
+      continue;
+    case 0:			/* timeout */
+      break;
+    default:			/* read! */
+      if(FD_ISSET(1, &readfd)) { /* read from stdin */
+        unsigned char outbuf[2];
+        int out_count = 0;
+        size_t bytes_written;
+        char *buffer = buf;
+        
+        nread = read(1, buf, 255);
 
-	 if(FD_ISSET(sockfd, &readfd))
-	 {
-#ifndef USE_SSLEAY
-	    nread = sread (sockfd, buf, BUFSIZE - 1);
-#else
-	    if (data->ssl.use) {
-	       nread = SSL_read (data->ssl.handle, buf, BUFSIZE - 1);
-	    }
-	    else {
-	       nread = sread (sockfd, buf, BUFSIZE - 1);
-	    }
-#endif /* USE_SSLEAY */
-	 }
-
-	 /* if we receive 0 or less here, the server closed the connection and
-	   we bail out from this! */
-	if (nread <= 0) {
-	  keepon = FALSE;
-	  break;
-	}
-
-	 telrcv(data, (unsigned char *)buf, nread);
+        while(nread--) {
+          outbuf[0] = *buffer++;
+          out_count = 1;
+          if(outbuf[0] == IAC)
+            outbuf[out_count++] = IAC;
+      
+          Curl_write(conn, data->firstsocket, outbuf,
+                     out_count, &bytes_written);
+        }
       }
-   }
-   return CURLE_OK;
+
+      if(FD_ISSET(sockfd, &readfd)) {
+        Curl_read(conn, sockfd, buf, BUFSIZE - 1, &nread);
+
+        /* if we receive 0 or less here, the server closed the connection and
+           we bail out from this! */
+        if (nread <= 0) {
+          keepon = FALSE;
+          break;
+        }
+
+        telrcv(data, (unsigned char *)buf, nread);
+      }
+    }
+  }
+  /* mark this as "no further transfer wanted" */
+  return Curl_Transfer(conn, -1, -1, FALSE, NULL, -1, NULL);
 }
 
 
