@@ -5,38 +5,21 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- *  The contents of this file are subject to the Mozilla Public License
- *  Version 1.0 (the "License"); you may not use this file except in
- *  compliance with the License. You may obtain a copy of the License at
- *  http://www.mozilla.org/MPL/
+ * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
- *  Software distributed under the License is distributed on an "AS IS"
- *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
- *  License for the specific language governing rights and limitations
- *  under the License.
+ * In order to be useful for every potential user, curl and libcurl are
+ * dual-licensed under the MPL and the MIT/X-derivate licenses.
  *
- *  The Original Code is Curl.
+ * You may opt to use, copy, modify, merge, publish, distribute and/or sell
+ * copies of the Software, and permit persons to whom the Software is
+ * furnished to do so, under the terms of the MPL or the MIT/X-derivate
+ * licenses. You may pick one of these licenses.
  *
- *  The Initial Developer of the Original Code is Daniel Stenberg.
+ * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+ * KIND, either express or implied.
  *
- *  Portions created by the Initial Developer are Copyright (C) 1998.
- *  All Rights Reserved.
- *
- * ------------------------------------------------------------
- * Main author:
- * - Daniel Stenberg <daniel@haxx.se>
- *
- * 	http://curl.haxx.se
- *
- * $Source: /cvsroot/curl/lib/url.c,v $
- * $Revision: 1.67 $
- * $Date: 2000/11/29 08:16:27 $
- * $Author: bagder $
- * $State: Exp $
- * $Locker:  $
- *
- * ------------------------------------------------------------
- ****************************************************************************/
+ * $Id: url.c,v 1.83 2001/02/13 13:34:16 bagder Exp $
+ *****************************************************************************/
 
 /* -- WIN32 approved -- */
 
@@ -97,14 +80,12 @@
 #include "netrc.h"
 
 #include "formdata.h"
-#include "getenv.h"
 #include "base64.h"
 #include "ssluse.h"
 #include "hostip.h"
 #include "if2ip.h"
-#include "download.h"
+#include "transfer.h"
 #include "sendf.h"
-#include "speedcheck.h"
 #include "getpass.h"
 #include "progress.h"
 #include "cookie.h"
@@ -241,7 +222,7 @@ void static urlfree(struct UrlData *data, bool totally)
       /* the URL is allocated, free it! */
       free(data->url);
 
-    cookie_cleanup(data->cookies);
+    Curl_cookie_cleanup(data->cookies);
 
     free(data);
 
@@ -265,6 +246,7 @@ CURLcode curl_close(CURL *curl)
   return CURLE_OK;
 }
 
+static
 int my_getpass(void *clientp, char *prompt, char* buffer, int buflen )
 {
   char *retbuf;
@@ -318,6 +300,8 @@ CURLcode curl_open(CURL **curl, char *url)
 
     data->current_speed = -1; /* init to negative == impossible */
 
+    data->httpreq = HTTPREQ_GET; /* Default HTTP request */
+
     *curl = data;
     return CURLE_OK;
   }
@@ -354,9 +338,14 @@ CURLcode curl_setopt(CURL *curl, CURLoption option, ...)
     break;
   case CURLOPT_UPLOAD:
     data->bits.upload = va_arg(param, long)?TRUE:FALSE;
+    if(data->bits.upload)
+      /* If this is HTTP, PUT is what's needed to "upload" */
+      data->httpreq = HTTPREQ_PUT;
     break;
   case CURLOPT_POST:
     data->bits.http_post = va_arg(param, long)?TRUE:FALSE;
+    if(data->bits.http_post)
+      data->httpreq = HTTPREQ_POST;
     break;
   case CURLOPT_FILETIME:
     data->bits.get_filetime = va_arg(param, long)?TRUE:FALSE;
@@ -378,19 +367,18 @@ CURLcode curl_setopt(CURL *curl, CURLoption option, ...)
     break;
   case CURLOPT_PUT:
     data->bits.http_put = va_arg(param, long)?TRUE:FALSE;
+    if(data->bits.http_put)
+      data->httpreq = HTTPREQ_PUT;
     break;
   case CURLOPT_MUTE:
     data->bits.mute = va_arg(param, long)?TRUE:FALSE;
     break;
-
   case CURLOPT_TIMECONDITION:
     data->timecondition = va_arg(param, long);
     break;
-
   case CURLOPT_TIMEVALUE:
     data->timevalue = va_arg(param, long);
     break;
-
   case CURLOPT_SSLVERSION:
     data->ssl.version = va_arg(param, long);
     break;
@@ -398,7 +386,7 @@ CURLcode curl_setopt(CURL *curl, CURLoption option, ...)
   case CURLOPT_COOKIEFILE:
     cookiefile = (char *)va_arg(param, void *);
     if(cookiefile) {
-      data->cookies = cookie_init(cookiefile);
+      data->cookies = Curl_cookie_init(cookiefile);
     }
     break;
   case CURLOPT_WRITEHEADER:
@@ -422,10 +410,14 @@ CURLcode curl_setopt(CURL *curl, CURLoption option, ...)
     break;
   case CURLOPT_CUSTOMREQUEST:
     data->customrequest = va_arg(param, char *);
+    if(data->customrequest)
+      data->httpreq = HTTPREQ_CUSTOM;
     break;
   case CURLOPT_HTTPPOST:
     data->httppost = va_arg(param, struct HttpPost *);
     data->bits.http_formpost = data->httppost?1:0;
+    if(data->bits.http_formpost)
+      data->httpreq = HTTPREQ_POST_FORM;
     break;
   case CURLOPT_INFILE:
     data->in = va_arg(param, FILE *);
@@ -550,50 +542,11 @@ CURLcode curl_setopt(CURL *curl, CURLoption option, ...)
   return CURLE_OK;
 }
 
-
-/*
- * Read everything until a newline.
- */
-
-int GetLine(int sockfd, char *buf, struct UrlData *data)
-{
-  int nread;
-  int read_rc=1;
-  char *ptr;
-  ptr=buf;
-
-  /* get us a full line, terminated with a newline */
-  for(nread=0;
-      (nread<BUFSIZE) && read_rc;
-      nread++, ptr++) {
-#ifdef USE_SSLEAY
-    if (data->ssl.use) {
-      read_rc = SSL_read(data->ssl.handle, ptr, 1);
-    }
-    else {
-#endif
-      read_rc = sread(sockfd, ptr, 1);
-#ifdef USE_SSLEAY
-    }
-#endif /* USE_SSLEAY */
-    if (*ptr == '\n')
-      break;
-  }
-  *ptr=0; /* zero terminate */
-
-  if(data->bits.verbose) {
-    fputs("< ", data->err);
-    fwrite(buf, 1, nread, data->err);
-    fputs("\n", data->err);
-  }
-  return nread;
-}
-
-
-#ifndef WIN32
+#if !defined(WIN32)||defined(__CYGWIN32__)
 #ifndef RETSIGTYPE
 #define RETSIGTYPE void
 #endif
+static
 RETSIGTYPE alarmfunc(int signal)
 {
   /* this is for "-ansi -Wall -pedantic" to stop complaining!   (rabe) */
@@ -602,75 +555,23 @@ RETSIGTYPE alarmfunc(int signal)
 }
 #endif
 
-CURLcode curl_write(CURLconnect *c_conn, char *buf, size_t amount,
-                   size_t *n)
-{
-  struct connectdata *conn = (struct connectdata *)c_conn;
-  struct UrlData *data;
-  size_t bytes_written;
-
-  if(!n || !conn || (conn->handle != STRUCT_CONNECT))
-    return CURLE_FAILED_INIT;
-  data = conn->data;
-
-#ifdef USE_SSLEAY
-  if (data->ssl.use) {
-    bytes_written = SSL_write(data->ssl.handle, buf, amount);
-  }
-  else {
-#endif
-#ifdef KRB4
-    if(conn->sec_complete)
-      bytes_written = sec_write(conn, conn->writesockfd, buf, amount);
-    else
-#endif
-      bytes_written = swrite(conn->writesockfd, buf, amount);
-#ifdef USE_SSLEAY
-  }
-#endif /* USE_SSLEAY */
-
-  *n = bytes_written;
-  return CURLE_OK;
-}
-
-CURLcode curl_read(CURLconnect *c_conn, char *buf, size_t buffersize,
-                   size_t *n)
-{
-  struct connectdata *conn = (struct connectdata *)c_conn;
-  struct UrlData *data;
-  size_t nread;
-
-  if(!n || !conn || (conn->handle != STRUCT_CONNECT))
-    return CURLE_FAILED_INIT;
-  data = conn->data;
-
-#ifdef USE_SSLEAY
-  if (data->ssl.use) {
-    nread = SSL_read (data->ssl.handle, buf, buffersize);
-  }
-  else {
-#endif
-#ifdef KRB4
-    if(conn->sec_complete)
-      nread = sec_read(conn, conn->sockfd, buf, buffersize);
-    else
-#endif
-      nread = sread (conn->sockfd, buf, buffersize);
-#ifdef USE_SSLEAY
-  }
-#endif /* USE_SSLEAY */
-  *n = nread;
-  return CURLE_OK;
-}
-
 CURLcode curl_disconnect(CURLconnect *c_connect)
 {
   struct connectdata *conn = c_connect;
 
   struct UrlData *data = conn->data;
 
+  if(data->proto.generic)
+    free(data->proto.generic);
+  data->proto.generic=NULL; /* it is gone */
+
+#ifdef ENABLE_IPV6
+  if(conn->res) /* host name info */
+    freeaddrinfo(conn->res);
+#else
   if(conn->hostent_buf) /* host name info */
     free(conn->hostent_buf);
+#endif
 
   if(conn->path) /* the URL path part */
     free(conn->path);
@@ -691,10 +592,18 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
   char resumerange[40]="";
   struct UrlData *data = curl;
   struct connectdata *conn;
+  char endbracket;
 #ifdef HAVE_SIGACTION
   struct sigaction sigact;
 #endif
   int urllen;
+#ifdef ENABLE_IPV6
+  struct addrinfo *ai;
+#endif
+
+  /*************************************************************
+   * Check input data
+   *************************************************************/
 
   if(!data || (data->handle != STRUCT_OPEN))
     return CURLE_BAD_FUNCTION_ARGUMENT; /* TBD: make error codes */
@@ -702,6 +611,9 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
   if(!data->url)
     return CURLE_URL_MALFORMAT;
 
+  /*************************************************************
+   * Allocate and initiate a connection struct
+   *************************************************************/
   conn = (struct connectdata *)malloc(sizeof(struct connectdata));
   if(!conn) {
     *in_connect = NULL; /* clear the pointer */
@@ -720,10 +632,16 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
 
   buf = data->buffer; /* this is our buffer */
 
+  /*************************************************************
+   * Set signal handler
+   *************************************************************/
 #ifdef HAVE_SIGACTION
   sigaction(SIGALRM, NULL, &sigact);
   sigact.sa_handler = alarmfunc;
+#ifdef SA_RESTART
+  /* HPUX doesn't have SA_RESTART but defaults to that behaviour! */
   sigact.sa_flags &= ~SA_RESTART;
+#endif
   sigaction(SIGALRM, &sigact, NULL);
 #else
   /* no sigaction(), revert to the much lamer signal() */
@@ -733,9 +651,11 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
 
 #endif
 
-  /* We need to allocate memory to store the path in. We get the size of the
-     full URL to be sure, and we need to make it at least 256 bytes since
-     other parts of the code will rely on this fact */
+  /***********************************************************
+   * We need to allocate memory to store the path in. We get the size of the
+   * full URL to be sure, and we need to make it at least 256 bytes since
+   * other parts of the code will rely on this fact
+   ***********************************************************/
 #define LEAST_PATH_ALLOC 256
   urllen=strlen(data->url);
   if(urllen < LEAST_PATH_ALLOC)
@@ -745,25 +665,30 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
   if(NULL == conn->path)
     return CURLE_OUT_OF_MEMORY; /* really bad error */
 
-  /* Parse <url> */
-  /* We need to parse the url, even when using the proxy, because
-   * we will need the hostname and port in case we are trying
-   * to SSL connect through the proxy -- and we don't know if we
-   * will need to use SSL until we parse the url ...
-   */
+  /*************************************************************
+   * Parse the URL.
+   *
+   * We need to parse the url even when using the proxy, because we will need
+   * the hostname and port in case we are trying to SSL connect through the
+   * proxy -- and we don't know if we will need to use SSL until we parse the
+   * url ...
+   ************************************************************/
   if((2 == sscanf(data->url, "%64[^:]://%[^\n]",
                   conn->proto,
                   conn->path)) && strequal(conn->proto, "file")) {
-    /* we deal with file://<host>/<path> differently since it
-       supports no hostname other than "localhost" and "127.0.0.1",
-       which is unique among the protocols specified in RFC 1738 */
+    /*
+     * we deal with file://<host>/<path> differently since it supports no
+     * hostname other than "localhost" and "127.0.0.1", which is unique among
+     * the URL protocols specified in RFC 1738
+     */
+
     if (strnequal(conn->path, "localhost/", 10) ||
         strnequal(conn->path, "127.0.0.1/", 10))
-      /* ... since coincidentally both host strings are of equal length
-         otherwise, <host>/ is quietly ommitted */
+      /* If there's another host name than the one we support, <host>/ is
+       * quietly ommitted */
       strcpy(conn->path, &conn->path[10]);
 
-    strcpy(conn->proto, "file");
+    strcpy(conn->proto, "file"); /* store protocol string lowercase */
   }
   else {
     /* Set default host and default path */
@@ -774,12 +699,24 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
                    "%64[^\n:]://%256[^\n/]%[^\n]",
                    conn->proto, conn->gname, conn->path)) {
       
-      /* badly formatted, let's try the browser-style _without_ 'http://' */
+      /*
+       * The URL was badly formatted, let's try the browser-style _without_
+       * protocol specified like 'http://'.
+       */
       if((1 > sscanf(data->url, "%256[^\n/]%[^\n]",
                      conn->gname, conn->path)) ) {
+        /*
+         * We couldn't even get this format.
+         */
         failf(data, "<url> malformed");
         return CURLE_URL_MALFORMAT;
       }
+
+      /*
+       * Since there was no protocol part specified, we guess what protocol it
+       * is based on the first letters of the server name.
+       */
+
       if(strnequal(conn->gname, "FTP", 3)) {
         strcpy(conn->proto, "ftp");
       }
@@ -803,6 +740,9 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
     }
   }
 
+  /*************************************************************
+   * Take care of user and password authentication stuff
+   *************************************************************/
 
   if(data->bits.user_passwd && !data->bits.use_netrc) {
     data->user[0] =0;
@@ -812,7 +752,7 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
       /* the name is given, get user+password */
       sscanf(data->userpwd, "%127[^:]:%127[^\n]",
              data->user, data->passwd);
-      }
+    }
     else
       /* no name given, get the password only */
       sscanf(data->userpwd+1, "%127[^\n]", data->passwd);
@@ -826,6 +766,9 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
     }
   }
 
+  /*************************************************************
+   * Take care of proxy authentication stuff
+   *************************************************************/
   if(data->bits.proxy_user_passwd) {
     data->proxyuser[0] =0;
     data->proxypasswd[0]=0;
@@ -851,37 +794,43 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
 
   }
 
+  /*************************************************************
+   * Set a few convenience pointers 
+   *************************************************************/
   conn->name = conn->gname;
   conn->ppath = conn->path;
   data->hostname = conn->name;
 
 
+  /*************************************************************
+   * Detect what (if any) proxy to use
+   *************************************************************/
   if(!data->bits.httpproxy) {
     /* If proxy was not specified, we check for default proxy environment
-       variables, to enable i.e Lynx compliance:
-
-       http_proxy=http://some.server.dom:port/
-       https_proxy=http://some.server.dom:port/
-       ftp_proxy=http://some.server.dom:port/
-       gopher_proxy=http://some.server.dom:port/
-       no_proxy=domain1.dom,host.domain2.dom
-                                 (a comma-separated list of hosts which should
-                                  not be proxied, or an asterisk to override
-                                  all proxy variables)
-       all_proxy=http://some.server.dom:port/
-                                 (seems to exist for the CERN www lib. Probably
-                                  the first to check for.)
-
-       For compatibility, the all-uppercase versions of these variables are
-       checked if the lowercase versions don't exist.
-       */
+     * variables, to enable i.e Lynx compliance:
+     *
+     * http_proxy=http://some.server.dom:port/
+     * https_proxy=http://some.server.dom:port/
+     * ftp_proxy=http://some.server.dom:port/
+     * gopher_proxy=http://some.server.dom:port/
+     * no_proxy=domain1.dom,host.domain2.dom
+     *   (a comma-separated list of hosts which should
+     *   not be proxied, or an asterisk to override
+     *   all proxy variables)
+     * all_proxy=http://some.server.dom:port/
+     *   (seems to exist for the CERN www lib. Probably
+     *   the first to check for.)
+     *
+     * For compatibility, the all-uppercase versions of these variables are
+     * checked if the lowercase versions don't exist.
+     */
     char *no_proxy=NULL;
     char *proxy=NULL;
     char proxy_env[128];
 
-    no_proxy=GetEnv("no_proxy");
+    no_proxy=curl_getenv("no_proxy");
     if(!no_proxy)
-      no_proxy=GetEnv("NO_PROXY");
+      no_proxy=curl_getenv("NO_PROXY");
 
     if(!no_proxy || !strequal("*", no_proxy)) {
       /* NO_PROXY wasn't specified or it wasn't just an asterisk */
@@ -913,22 +862,22 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
 	strcpy(envp, "_proxy");
 
 	/* read the protocol proxy: */
-	prox=GetEnv(proxy_env);
+	prox=curl_getenv(proxy_env);
 
 	if(!prox) {
           /* There was no lowercase variable, try the uppercase version: */
 	  for(envp = proxy_env; *envp; envp++)
 	    *envp = toupper(*envp);
-	  prox=GetEnv(proxy_env);
+	  prox=curl_getenv(proxy_env);
 	}
 
 	if(prox && *prox) { /* don't count "" strings */
 	  proxy = prox; /* use this */
 	}
 	else {
-	  proxy = GetEnv("all_proxy"); /* default proxy to use */
+	  proxy = curl_getenv("all_proxy"); /* default proxy to use */
 	  if(!proxy)
-            proxy=GetEnv("ALL_PROXY");
+            proxy=curl_getenv("ALL_PROXY");
 	}
 
         if(proxy && *proxy) {
@@ -943,13 +892,16 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
       free(no_proxy);
   } /* if not using proxy */
 
+  /*************************************************************
+   * No protocol but proxy usage needs attention
+   *************************************************************/
   if((conn->protocol&PROT_MISSING) && data->bits.httpproxy ) {
     /* We're guessing prefixes here and since we're told to use a proxy, we
        need to add the protocol prefix to the URL string before we continue!
        */
     char *reurl;
 
-    reurl = maprintf("%s://%s", conn->proto, data->url);
+    reurl = aprintf("%s://%s", conn->proto, data->url);
 
     if(!reurl)
       return CURLE_OUT_OF_MEMORY;
@@ -962,13 +914,15 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
     conn->protocol &= ~PROT_MISSING; /* switch that one off again */
   }
 
-  /* RESUME on a HTTP page is a tricky business. First, let's just check that
-     'range' isn't used, then set the range parameter and leave the resume as
-     it is to inform about this situation for later use. We will then
-     "attempt" to resume, and if we're talking to a HTTP/1.1 (or later)
-     server, we will get the document resumed. If we talk to a HTTP/1.0
-     server, we just fail since we can't rewind the file writing from within
-     this function. */
+  /************************************************************
+   * RESUME on a HTTP page is a tricky business. First, let's just check that
+   * 'range' isn't used, then set the range parameter and leave the resume as
+   * it is to inform about this situation for later use. We will then
+   * "attempt" to resume, and if we're talking to a HTTP/1.1 (or later)
+   * server, we will get the document resumed. If we talk to a HTTP/1.0
+   * server, we just fail since we can't rewind the file writing from within
+   * this function.
+   ***********************************************************/
   if(data->resume_from) {
     if(!data->bits.set_range) {
       /* if it already was in use, we just skip this */
@@ -979,28 +933,28 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
     }
   }
 
-
+  /*************************************************************
+   * Set timeout if that is being used
+   *************************************************************/
   if(data->timeout) {
     /* We set the timeout on the connection/resolving phase first, separately
-       from the download/upload part to allow a maximum time on everything */
+     * from the download/upload part to allow a maximum time on everything */
     myalarm(data->timeout); /* this sends a signal when the timeout fires
 			       off, and that will abort system calls */
   }
 
-  /*
-   * Hmm, if we are using a proxy, then we can skip the GOPHER and the
-   * FTP steps, although we cannot skip the HTTPS step (since the proxy
-   * works differently, depending on whether its SSL or not).
-   */
+  /*************************************************************
+   * Setup internals depending on protocol
+   *************************************************************/
 
   if (strequal(conn->proto, "HTTP")) {
     if(!data->port)
       data->port = PORT_HTTP;
     data->remote_port = PORT_HTTP;
     conn->protocol |= PROT_HTTP;
-    conn->curl_do = http;
-    conn->curl_done = http_done;
-    conn->curl_close = http_close;
+    conn->curl_do = Curl_http;
+    conn->curl_done = Curl_http_done;
+    conn->curl_close = Curl_http_close;
   }
   else if (strequal(conn->proto, "HTTPS")) {
 #ifdef USE_SSLEAY
@@ -1010,10 +964,10 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
     conn->protocol |= PROT_HTTP;
     conn->protocol |= PROT_HTTPS;
 
-    conn->curl_do = http;
-    conn->curl_done = http_done;
-    conn->curl_connect = http_connect;
-    conn->curl_close = http_close;
+    conn->curl_do = Curl_http;
+    conn->curl_done = Curl_http_done;
+    conn->curl_connect = Curl_http_connect;
+    conn->curl_close = Curl_http_close;
 
 #else /* USE_SSLEAY */
     failf(data, "libcurl was built with SSL disabled, https: not supported!");
@@ -1031,9 +985,9 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
 	conn->ppath = conn->path;
       }
     conn->protocol |= PROT_GOPHER;
-    conn->curl_do = http;
-    conn->curl_done = http_done;
-    conn->curl_close = http_close;
+    conn->curl_do = Curl_http;
+    conn->curl_done = Curl_http_done;
+    conn->curl_close = Curl_http_close;
   }
   else if(strequal(conn->proto, "FTP")) {
     char *type;
@@ -1046,20 +1000,20 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
        !data->bits.tunnel_thru_httpproxy) {
       /* Unless we have asked to tunnel ftp operations through the proxy, we
          switch and use HTTP operations only */
-      conn->curl_do = http;
-      conn->curl_done = http_done;
-      conn->curl_close = http_close;
+      conn->curl_do = Curl_http;
+      conn->curl_done = Curl_http_done;
+      conn->curl_close = Curl_http_close;
     }
     else {
-      conn->curl_do = ftp;
-      conn->curl_done = ftp_done;
-      conn->curl_connect = ftp_connect;
+      conn->curl_do = Curl_ftp;
+      conn->curl_done = Curl_ftp_done;
+      conn->curl_connect = Curl_ftp_connect;
     }
 
     conn->ppath++; /* don't include the initial slash */
 
     /* FTP URLs support an extension like ";type=<typecode>" that
-       we'll try to get now! */
+     * we'll try to get now! */
     type=strstr(conn->ppath, ";type=");
     if(!type) {
       type=strstr(conn->gname, ";type=");
@@ -1090,8 +1044,8 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
       data->port = PORT_TELNET;
     data->remote_port = PORT_TELNET;
 
-    conn->curl_do = telnet;
-    conn->curl_done = telnet_done;
+    conn->curl_do = Curl_telnet;
+    conn->curl_done = Curl_telnet_done;
 
   }
   else if (strequal(conn->proto, "DICT")) {
@@ -1099,16 +1053,16 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
     if(!data->port)
       data->port = PORT_DICT;
     data->remote_port = PORT_DICT;
-    conn->curl_do = dict;
-    conn->curl_done = dict_done;
+    conn->curl_do = Curl_dict;
+    conn->curl_done = Curl_dict_done;
   }
   else if (strequal(conn->proto, "LDAP")) {
     conn->protocol |= PROT_LDAP;
     if(!data->port)
       data->port = PORT_LDAP;
     data->remote_port = PORT_LDAP;
-    conn->curl_do = ldap;
-    conn->curl_done = ldap_done;
+    conn->curl_do = Curl_ldap;
+    conn->curl_done = Curl_ldap_done;
   }
   else if (strequal(conn->proto, "FILE")) {
     conn->protocol |= PROT_FILE;
@@ -1116,38 +1070,42 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
     conn->curl_do = file;
     /* no done() function */
 
-    result = Transfer(conn, -1, -1, FALSE, NULL, /* no download */
+    result = Curl_Transfer(conn, -1, -1, FALSE, NULL, /* no download */
                       -1, NULL); /* no upload */
 
     return CURLE_OK;
   }
-
   else {
+    /* We fell through all checks and thus we don't support the specified
+       protocol */
     failf(data, "Unsupported protocol: %s", conn->proto);
     return CURLE_UNSUPPORTED_PROTOCOL;
   }
 
+  /*************************************************************
+   * .netrc scanning coming up
+   *************************************************************/
   if(data->bits.use_netrc) {
-    if(ParseNetrc(data->hostname, data->user, data->passwd)) {
+    if(Curl_parsenetrc(data->hostname, data->user, data->passwd)) {
       infof(data, "Couldn't find host %s in the .netrc file, using defaults",
             data->hostname);
     }
+    else
+      data->bits.user_passwd = 1; /* enable user+password */
+
     /* weather we failed or not, we don't know which fields that were filled
        in anyway */
     if(!data->user[0])
       strcpy(data->user, CURL_DEFAULT_USER);
     if(!data->passwd[0])
       strcpy(data->passwd, CURL_DEFAULT_PASSWORD);
-    if(conn->protocol&PROT_HTTP) {
-      data->bits.user_passwd = 1; /* enable user+password */
-    }
   }
   else if(!(data->bits.user_passwd) &&
 	  (conn->protocol & (PROT_FTP|PROT_HTTP)) ) {
     /* This is a FTP or HTTP URL, and we haven't got the user+password in
-       the extra parameter, we will now try to extract the possible
-       user+password pair in a string like:
-       ftp://user:password@ftp.my.site:8021/README */
+     * the extra parameter, we will now try to extract the possible
+     * user+password pair in a string like:
+     * ftp://user:password@ftp.my.site:8021/README */
     char *ptr=NULL; /* assign to remove possible warnings */
     if((ptr=strchr(conn->name, '@'))) {
       /* there's a user+password given here, to the left of the @ */
@@ -1199,24 +1157,66 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
     }
   }
 
+  /*************************************************************
+   * Figure out the remote port number
+   *
+   * No matter if we use a proxy or not, we have to figure out the remote
+   * port number of various reasons.
+   *
+   * To be able to detect port number flawlessly, we must not confuse them
+   * IPv6-specified addresses in the [0::1] style.
+   *************************************************************/
+
+  if((1 == sscanf(conn->name, "[%*39[0-9a-fA-F:]%c", &endbracket)) &&
+     (']' == endbracket)) {
+    /* this is a IPv6-style specified IP-address */
+#ifndef ENABLE_IPV6
+    failf(data, "You haven't enabled IPv6 support");
+    return CURLE_URL_MALFORMAT;
+#else
+    tmp = strchr(conn->name, ']');
+
+    tmp++; /* pass the ending bracket */
+    if(':' != *tmp)
+      tmp = NULL; /* no port number available */
+#endif
+  }
+  else {
+    /* traditional IPv4-style port-extracting */
+    tmp = strchr(conn->name, ':');
+  }
+
+  if (tmp) {
+    *tmp++ = '\0'; /* cut off the name there */
+    data->remote_port = atoi(tmp);
+  }
+
+  /*************************************************************
+   * Resolve the name of the server or proxy
+   *************************************************************/
   if(!data->bits.httpproxy) {
     /* If not connecting via a proxy, extract the port from the URL, if it is
      * there, thus overriding any defaults that might have been set above. */
-    tmp = strchr(conn->name, ':');
-    if (tmp) {
-      *tmp++ = '\0';
-      data->port = atoi(tmp);
-    }
-    data->remote_port = data->port; /* it is the same port */
+    data->port =  data->remote_port; /* it is the same port */
 
     /* Connect to target host right on */
-    conn->hp = GetHost(data, conn->name, &conn->hostent_buf);
-    if(!conn->hp) {
+#ifdef ENABLE_IPV6
+    conn->res = Curl_getaddrinfo(data, conn->name, data->port);
+    if(!conn->res)
+#else
+    conn->hp = Curl_gethost(data, conn->name, &conn->hostent_buf);
+    if(!conn->hp)
+#endif
+    {
       failf(data, "Couldn't resolve host '%s'", conn->name);
       return CURLE_COULDNT_RESOLVE_HOST;
     }
   }
   else {
+#ifdef ENABLE_IPV6
+    failf(data, "proxy yet to be supported");
+    return CURLE_OUT_OF_MEMORY;
+#else
     char *prox_portno;
     char *endofprot;
 
@@ -1232,14 +1232,6 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
       return CURLE_OUT_OF_MEMORY;
     }
 
-    /* we use proxy all right, but we wanna know the remote port for SSL
-       reasons */
-    tmp = strchr(conn->name, ':');
-    if (tmp) {
-      *tmp++ = '\0'; /* cut off the name there */
-      data->remote_port = atoi(tmp);
-    }
-
     /* Daniel Dec 10, 1998:
        We do the proxy host string parsing here. We want the host name and the
        port name. Accept a protocol:// prefix, even though it should just be
@@ -1266,16 +1258,18 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
     }
 
     /* connect to proxy */
-    conn->hp = GetHost(data, proxyptr, &conn->hostent_buf);
+    conn->hp = Curl_gethost(data, proxyptr, &conn->hostent_buf);
     if(!conn->hp) {
       failf(data, "Couldn't resolve proxy '%s'", proxyptr);
       return CURLE_COULDNT_RESOLVE_PROXY;
     }
 
     free(proxydup); /* free the duplicate pointer and not the modified */
+#endif
   }
-  pgrsTime(data, TIMER_NAMELOOKUP);
+  Curl_pgrsTime(data, TIMER_NAMELOOKUP);
 
+#ifndef ENABLE_IPV6
   data->firstsocket = socket(AF_INET, SOCK_STREAM, 0);
 
   memset((char *) &conn->serv_addr, '\0', sizeof(conn->serv_addr));
@@ -1283,21 +1277,23 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
          conn->hp->h_addr, conn->hp->h_length);
   conn->serv_addr.sin_family = conn->hp->h_addrtype;
   conn->serv_addr.sin_port = htons(data->port);
+#endif
 
-#ifndef WIN32 
+#if !defined(WIN32)||defined(__CYGWIN32__)
   /* We don't generally like checking for OS-versions, we should make this
      HAVE_XXXX based, although at the moment I don't have a decent test for
      this! */
 
-  /* sck 8/31/2000 add support for specifing device to bind socket to */
-  /* I am using this, but it may not work everywhere, only tested on
-     RedHat 6.2 */
 #ifdef HAVE_INET_NTOA
 
 #ifndef INADDR_NONE
 #define INADDR_NONE (unsigned long) ~0
 #endif
 
+#ifndef ENABLE_IPV6
+  /*************************************************************
+   * Select device to bind socket to
+   *************************************************************/
   if (data->device && (strlen(data->device)<255)) {
     struct sockaddr_in sa;
     struct hostent *h=NULL;
@@ -1306,12 +1302,12 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
     char myhost[256] = "";
     unsigned long in;
 
-    if(if2ip(data->device, myhost, sizeof(myhost))) {
-      h = GetHost(data, myhost, &hostdataptr);
+    if(Curl_if2ip(data->device, myhost, sizeof(myhost))) {
+      h = Curl_gethost(data, myhost, &hostdataptr);
     }
     else {
       if(strlen(data->device)>1) {
-        h = GetHost(data, data->device, &hostdataptr);
+        h = Curl_gethost(data, data->device, &hostdataptr);
       }
       if(h) {
         /* we know data->device is shorter than the myhost array */
@@ -1321,7 +1317,7 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
 
     if(! *myhost) {
       /* need to fix this
-         h=GetHost(data,
+         h=Curl_gethost(data,
          getmyhost(*myhost,sizeof(myhost)),
          hostent_buf,
          sizeof(hostent_buf));
@@ -1398,12 +1394,36 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
     }
 
     if(hostdataptr)
-      free(hostdataptr); /* allocated by GetHost() */
+      free(hostdataptr); /* allocated by Curl_gethost() */
 
   } /* end of device selection support */
 #endif  /* end of HAVE_INET_NTOA */
 #endif /* end of not WIN32 */
+#endif /*ENABLE_IPV6*/
 
+  /*************************************************************
+   * Connect to server/proxy
+   *************************************************************/
+#ifdef ENABLE_IPV6
+  data->firstsocket = -1;
+  for (ai = conn->res; ai; ai = ai->ai_next) {
+    data->firstsocket = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
+    if (data->firstsocket < 0)
+      continue;
+
+    if (connect(data->firstsocket, ai->ai_addr, ai->ai_addrlen) < 0) {
+      close(data->firstsocket);
+      data->firstsocket = -1;
+      continue;
+    }
+
+    break;
+  }
+  if (data->firstsocket < 0) {
+    failf(data, strerror(errno));
+    return CURLE_COULDNT_CONNECT;
+  }
+#else
   if (connect(data->firstsocket,
               (struct sockaddr *) &(conn->serv_addr),
               sizeof(conn->serv_addr)
@@ -1452,45 +1472,79 @@ static CURLcode _connect(CURL *curl, CURLconnect **in_connect)
     }
     return CURLE_COULDNT_CONNECT;
   }
+#endif
 
+  /*************************************************************
+   * Proxy authentication
+   *************************************************************/
   if(data->bits.proxy_user_passwd) {
     char *authorization;
     snprintf(data->buffer, BUFSIZE, "%s:%s",
              data->proxyuser, data->proxypasswd);
-    if(base64_encode(data->buffer, strlen(data->buffer),
-                    &authorization) >= 0) {
+    if(Curl_base64_encode(data->buffer, strlen(data->buffer),
+                          &authorization) >= 0) {
       data->ptr_proxyuserpwd =
-        maprintf("Proxy-authorization: Basic %s\015\012", authorization);
+        aprintf("Proxy-authorization: Basic %s\015\012", authorization);
       free(authorization);
     }
   }
+
+  /*************************************************************
+   * Send user-agent to HTTP proxies even if the target protocol
+   * isn't HTTP.
+   *************************************************************/
   if((conn->protocol&PROT_HTTP) || data->bits.httpproxy) {
     if(data->useragent) {
-      data->ptr_uagent = maprintf("User-Agent: %s\015\012", data->useragent);
+      data->ptr_uagent =
+        aprintf("User-Agent: %s\015\012", data->useragent);
     }
   }
 
   if(conn->curl_connect) {
     /* is there a connect() procedure? */
-    conn->now = tvnow(); /* set this here for timeout purposes in the
-                            connect procedure, it is later set again for the
-                            progress meter purpose */
+
+    /* set start time here for timeout purposes in the
+     * connect procedure, it is later set again for the
+     * progress meter purpose */
+    conn->now = Curl_tvnow();
+
+    /* Call the protocol-specific connect function */
     result = conn->curl_connect(conn);
     if(result != CURLE_OK)
       return result; /* pass back errors */
   }
 
-  pgrsTime(data, TIMER_CONNECT); /* we're connected */
+  Curl_pgrsTime(data, TIMER_CONNECT); /* we're connected */
 
-  conn->now = tvnow(); /* time this *after* the connect is done */
+  conn->now = Curl_tvnow(); /* time this *after* the connect is done */
   conn->bytecount = 0;
   
-  /* Figure out the ip-number and the first host name it shows: */
+  /* Figure out the ip-number and display the first host name it shows: */
+#ifdef ENABLE_IPV6
+  {
+    char hbuf[NI_MAXHOST];
+#ifdef NI_WITHSCOPEID
+    const int niflags = NI_NUMERICHOST | NI_WITHSCOPEID;
+#else
+    const int niflags = NI_NUMERICHOST;
+#endif
+    if (getnameinfo(ai->ai_addr, ai->ai_addrlen, hbuf, sizeof(hbuf), NULL, 0,
+	niflags)) {
+      snprintf(hbuf, sizeof(hbuf), "?");
+    }
+    if (ai->ai_canonname) {
+      infof(data, "Connected to %s (%s)\n", ai->ai_canonname, hbuf);
+    } else {
+      infof(data, "Connected to %s\n", hbuf);
+    }
+  }
+#else
   {
     struct in_addr in;
     (void) memcpy(&in.s_addr, *conn->hp->h_addr_list, sizeof (in.s_addr));
     infof(data, "Connected to %s (%s)\n", conn->hp->h_name, inet_ntoa(in));
   }
+#endif
 
 #ifdef __EMX__
   /* 20000330 mgs
@@ -1522,8 +1576,13 @@ CURLcode curl_connect(CURL *curl, CURLconnect **in_connect)
     if(conn) {
       if(conn->path)
         free(conn->path);
+#ifdef ENABLE_IPV6
+      if(conn->res)
+        freeaddrinfo(conn->res);
+#else
       if(conn->hostent_buf)
         free(conn->hostent_buf);
+#endif
       free(conn);
       *in_connect=NULL;
     }
@@ -1574,7 +1633,7 @@ CURLcode curl_done(CURLconnect *c_connect)
   else
     result = CURLE_OK;
 
-  pgrsDone(data); /* done with the operation */
+  Curl_pgrsDone(data); /* done with the operation */
 
   conn->state = CONN_DONE;
 
@@ -1589,7 +1648,13 @@ CURLcode curl_do(CURLconnect *in_conn)
   if(!conn || (conn->handle!= STRUCT_CONNECT)) {
     return CURLE_BAD_FUNCTION_ARGUMENT;
   }
-  if(conn->state != CONN_INIT) {
+  switch(conn->state) {
+  case CONN_INIT:
+  case CONN_DONE:
+    /* these two states are OK */
+    break;
+  default:
+    /* anything else is bad */
     return CURLE_BAD_CALLING_ORDER;
   }
 
