@@ -7,38 +7,21 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- *  The contents of this file are subject to the Mozilla Public License
- *  Version 1.0 (the "License"); you may not use this file except in
- *  compliance with the License. You may obtain a copy of the License at
- *  http://www.mozilla.org/MPL/
+ * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
- *  Software distributed under the License is distributed on an "AS IS"
- *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
- *  License for the specific language governing rights and limitations
- *  under the License.
+ * In order to be useful for every potential user, curl and libcurl are
+ * dual-licensed under the MPL and the MIT/X-derivate licenses.
  *
- *  The Original Code is Curl.
+ * You may opt to use, copy, modify, merge, publish, distribute and/or sell
+ * copies of the Software, and permit persons to whom the Software is
+ * furnished to do so, under the terms of the MPL or the MIT/X-derivate
+ * licenses. You may pick one of these licenses.
  *
- *  The Initial Developer of the Original Code is Daniel Stenberg.
+ * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+ * KIND, either express or implied.
  *
- *  Portions created by the Initial Developer are Copyright (C) 1998.
- *  All Rights Reserved.
- *
- * ------------------------------------------------------------
- * Main author:
- * - Daniel Stenberg <daniel@haxx.se>
- *
- * 	http://curl.haxx.se
- *
- * $Source: /cvsroot/curl/lib/urldata.h,v $
- * $Revision: 1.35 $
- * $Date: 2000/11/28 09:05:47 $
- * $Author: bagder $
- * $State: Exp $
- * $Locker:  $
- *
- * ------------------------------------------------------------
- ****************************************************************************/
+ * $Id: urldata.h,v 1.40 2001/02/05 23:04:44 bagder Exp $
+ *****************************************************************************/
 
 /* This file is for lib internal stuff */
 
@@ -99,8 +82,8 @@
 /* Download buffer size, keep it fairly big for speed reasons */
 #define BUFSIZE (1024*50)
 
-/* Upload buffer size, keep it smallish to get faster progress meter
-   updates. This should probably become dynamic and adjust to the upload
+/* Defaul upload buffer size, keep it smallish to get faster progress meter
+   updates. This is just default, it is dynamic and adjusts to the upload
    speed. */
 #define UPLOAD_BUFSIZE (1024*2)
 
@@ -108,10 +91,14 @@
    of need. */
 #define HEADERSIZE 256
 
+/* Just a convenience macro to get the larger value out of two given */
 #ifndef MAX
 #define MAX(x,y) ((x)>(y)?(x):(y))
 #endif
 
+/* Type of handle. All publicly returned 'handles' in the curl interface
+   have a handle first in the struct that describes what kind of handle it
+   is. Used to detect bad handle usage. */
 typedef enum {
   STRUCT_NONE,
   STRUCT_OPEN,
@@ -119,6 +106,8 @@ typedef enum {
   STRUCT_LAST
 } Handle;
 
+/* Connecting to a remote server using the curl interface is moving through
+   a state machine, this type is used to store the current state */
 typedef enum {
   CONN_NONE,  /* illegal state */
   CONN_INIT,  /* curl_connect() has been called */
@@ -129,6 +118,7 @@ typedef enum {
 } ConnState;
 
 #ifdef KRB4
+/* Types needed for krb4-ftp connections */
 struct krb4buffer {
   void *data;
   size_t size;
@@ -169,16 +159,20 @@ struct connectdata {
 #define PROT_LDAP    (1<<7)
 #define PROT_FILE    (1<<8)
 
+#ifdef ENABLE_IPV6
+  struct addrinfo *res;
+#else
   char *hostent_buf; /* pointer to allocated memory for name info */
   struct hostent *hp;
   struct sockaddr_in serv_addr;
-  char proto[64];
-  char gname[256];
-  char *name;
-  char *path; /* formerly staticly this size: URL_MAX_LENGTH */
+#endif
+  char proto[64];  /* store the protocol string in this buffer */
+  char gname[257]; /* store the hostname in this buffer */
+  char *name;      /* host name pointer to fool around with */
+  char *path;      /* allocated buffer to store the URL's path part in */
   char *ppath;
   long bytecount;
-  struct timeval now;
+  struct timeval now; /* current time */
 
   long upload_bufsize; /* adjust as you see fit, never bigger than BUFSIZE
                           never smaller than UPLOAD_BUFSIZE */
@@ -265,9 +259,9 @@ struct Progress {
 struct HTTP {
   struct FormData *sendit;
   int postsize;
-  char *p_pragma;
-  char *p_accept;
-  long readbytecount;
+  char *p_pragma;      /* Pragma: string */
+  char *p_accept;      /* Accept: string */
+  long readbytecount; 
   long writebytecount;
 
   /* For FORM posting */
@@ -281,14 +275,32 @@ struct HTTP {
  ***************************************************************************/
 struct FTP {
   long *bytecountp;
-  char *user;
-  char *passwd;
+  char *user;    /* user name string */
+  char *passwd;  /* password string */
   char *urlpath; /* the originally given path part of the URL */
   char *dir;     /* decoded directory */
   char *file;    /* decoded file */
 };
 
+typedef enum {
+  HTTPREQ_NONE, /* first in list */
+  HTTPREQ_GET,
+  HTTPREQ_POST,
+  HTTPREQ_POST_FORM, /* we make a difference internally */
+  HTTPREQ_PUT,
+  HTTPREQ_CUSTOM,
+  HTTPREQ_LAST /* last in list */
+} Curl_HttpReq;
+
+/* This struct is for boolean settings that define how to behave during
+   this session. */
 struct Configbits {
+  /* these four request types mirror the httpreq field */
+  bool http_formpost;
+  bool http_post;
+  bool http_put;
+  bool http_get;
+
   bool get_filetime;
   bool tunnel_thru_httpproxy;
   bool ftp_append;
@@ -298,10 +310,7 @@ struct Configbits {
   bool hide_progress;
   bool http_fail_on_error;
   bool http_follow_location;
-  bool http_formpost;
   bool http_include_header;
-  bool http_post;
-  bool http_put;
   bool http_set_referer;
   bool http_auto_referer; /* set "correct" referer when following location: */
   bool httpproxy;
@@ -316,7 +325,6 @@ struct Configbits {
   bool verbose;
   bool this_is_a_follow; /* this is a followed Location: request */
   bool krb4; /* kerberos4 connection requested */
-
   bool proxystringalloc; /* the http proxy string is malloc()'ed */
   bool rangestringalloc; /* the range string is malloc()'ed */
   bool urlstringalloc;   /* the URL string is malloc()'ed */
@@ -330,6 +338,7 @@ typedef enum {
   CURLI_LAST
 } CurlInterface;
 
+/* struct for data related to SSL and SSL connections */
 struct ssldata {
   bool use;              /* use ssl encrypted communications TRUE/FALSE */
   long version;          /* what version the client wants to use */
@@ -485,8 +494,10 @@ struct UrlData {
   struct curl_slist *quote;     /* before the transfer */
   struct curl_slist *postquote; /* after the transfer */
 
-  TimeCond timecondition;
-  time_t timevalue;
+  TimeCond timecondition; /* kind of comparison */
+  time_t timevalue;       /* what time to compare with */
+
+  Curl_HttpReq httpreq; /* what kind of HTTP request (if any) is this */
 
   char *customrequest; /* http/ftp request to use */
 
@@ -499,7 +510,7 @@ struct UrlData {
                        completion */
 #endif
 
-  struct Progress progress;
+  struct Progress progress; /* for all the progress meter data */
 
 #define MAX_CURL_USER_LENGTH 128
 #define MAX_CURL_PASSWORD_LENGTH 128
@@ -509,6 +520,7 @@ struct UrlData {
                     * host (which location-following otherwise could lead to)
                     */
 
+  /* buffers to store authentication data in */
   char user[MAX_CURL_USER_LENGTH];
   char passwd[MAX_CURL_PASSWORD_LENGTH];
   char proxyuser[MAX_CURL_USER_LENGTH];
@@ -523,7 +535,7 @@ struct UrlData {
   char *ptr_cookie; /* free later if not NULL! */
   char *ptr_host; /* free later if not NULL */
 
-  char *krb4_level;
+  char *krb4_level; /* what security level */
 #ifdef KRB4
   FILE *cmdchannel;
 #endif
