@@ -5,38 +5,21 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- *  The contents of this file are subject to the Mozilla Public License
- *  Version 1.0 (the "License"); you may not use this file except in
- *  compliance with the License. You may obtain a copy of the License at
- *  http://www.mozilla.org/MPL/
+ * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
- *  Software distributed under the License is distributed on an "AS IS"
- *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
- *  License for the specific language governing rights and limitations
- *  under the License.
+ * In order to be useful for every potential user, curl and libcurl are
+ * dual-licensed under the MPL and the MIT/X-derivate licenses.
  *
- *  The Original Code is Curl.
+ * You may opt to use, copy, modify, merge, publish, distribute and/or sell
+ * copies of the Software, and permit persons to whom the Software is
+ * furnished to do so, under the terms of the MPL or the MIT/X-derivate
+ * licenses. You may pick one of these licenses.
  *
- *  The Initial Developer of the Original Code is Daniel Stenberg.
+ * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+ * KIND, either express or implied.
  *
- *  Portions created by the Initial Developer are Copyright (C) 1998.
- *  All Rights Reserved.
- *
- * ------------------------------------------------------------
- * Main author:
- * - Daniel Stenberg <daniel@haxx.se>
- *
- * 	http://curl.haxx.se
- *
- * $Source: /cvsroot/curl/src/main.c,v $
- * $Revision: 1.52 $
- * $Date: 2000/11/28 09:10:43 $
- * $Author: bagder $
- * $State: Exp $
- * $Locker:  $
- *
- * ------------------------------------------------------------
- ****************************************************************************/
+ * $Id: main.c,v 1.68 2001/01/29 07:26:50 bagder Exp $
+ *****************************************************************************/
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -56,7 +39,6 @@
 #include "writeout.h"
 
 #define CURLseparator	"--_curl_--"
-#define MIMEseparator	"_curl_"
 
 /* This define make use of the "Curlseparator" as opposed to the
    MIMEseparator. We might add support for the latter one in the
@@ -66,7 +48,7 @@
 /* This is now designed to have its own local setup.h */
 #include "setup.h"
 
-#ifdef WIN32
+#if defined(WIN32)&&!defined(__CYGWIN32__)
 #include <winsock.h>
 #endif
 
@@ -127,7 +109,9 @@ typedef enum {
 #define CONF_FOLLOWLOCATION (1<<23) /* use Location: Luke! */
 #define CONF_GETTEXT  (1<<24) /* use ASCII/text for transfer */
 #define CONF_HTTPPOST (1<<25) /* multipart/form-data HTTP POST */
+#if 0
 #define CONF_PUT      (1<<27) /* PUT the input file */
+#endif
 #define CONF_MUTE     (1<<28) /* force NOPROGRESS */
 
 #ifndef HAVE_STRDUP
@@ -239,6 +223,20 @@ static void helpf(char *fmt, ...)
   fprintf(stderr, "curl: try 'curl --help' for more information\n");
 }
 
+/*
+ * A chain of these nodes contain URL to get and where to put the URL's
+ * contents.
+ */
+struct getout {
+  struct getout *next;
+  char *url;
+  char *outfile;
+  int flags;
+};
+#define GETOUT_OUTFILE (1<<0) /* set when outfile is deemed done */
+#define GETOUT_URL     (1<<1) /* set when URL is deemed done */
+#define GETOUT_USEREMOTE (1<<2) /* use remote file name locally */
+
 static void help(void)
 {
   printf(CURL_ID "%s\n"
@@ -248,7 +246,6 @@ static void help(void)
        " -A/--user-agent <string> User-Agent to send to server (H)\n"
        " -b/--cookie <name=string/file> Cookie string or file to read cookies from (H)\n"
        " -B/--use-ascii     Use ASCII/text transfer\n"
-       " -c/--continue      Resume a previous transfer where we left it\n"
        " -C/--continue-at <offset> Specify absolute resume offset\n"
        " -d/--data <data>   HTTP POST data (H)\n"
        "    --data-ascii <data>   HTTP POST ASCII data (H)\n"
@@ -259,7 +256,7 @@ static void help(void)
        "    --cacert <file> CA certifciate to verify peer against (HTTPS)\n"
        " -f/--fail          Fail silently (no output at all) on errors (H)\n"
        " -F/--form <name=content> Specify HTTP POST data (H)\n"
-
+       " -g/--globoff       Disable URL sequences and ranges using {} and []\n"
        " -h/--help          This help text\n"
        " -H/--header <line> Custom header to pass to server. (H)\n"
        " -i/--include       Include the HTTP-header in the output (H)\n"
@@ -282,8 +279,8 @@ static void help(void)
        " -r/--range <range> Retrieve a byte range from a HTTP/1.1 or FTP server\n"
        " -s/--silent        Silent mode. Don't output anything\n"
        " -S/--show-error    Show error. With -s, make curl show errors when they occur\n"
-       " -t/--upload        Transfer/upload stdin to remote site\n"
        " -T/--upload-file <file> Transfer/upload <file> to remote site\n"
+       "    --url <URL>     Another way to specify URL to work with\n"
        " -u/--user <user[:password]> Specify user and password to use\n"
        " -U/--proxy-user <user[:password]> Specify Proxy authentication\n"
        " -v/--verbose       Makes the operation more talkative\n"
@@ -320,9 +317,7 @@ struct Configurable {
   char *referer;
   long timeout;
   long maxredirs;
-  char *outfile;
   char *headerfile;
-  char remotefile;
   char *ftpport;
   char *iface;
   unsigned short porttouse;
@@ -337,7 +332,13 @@ struct Configurable {
   bool configread;
   bool proxytunnel;
   long conf;
-  char *url;
+
+  struct getout *url_list; /* point to the first node */
+  struct getout *url_last; /* point to the last/current node */
+
+  struct getout *url_get;  /* point to the node to fill in URL */
+  struct getout *url_out;  /* point to the node to fill in outfile */
+
   char *cert;
   char *cacert;
   char *cert_passwd;
@@ -347,6 +348,7 @@ struct Configurable {
   char *krb4level;
   bool progressmode;
   bool nobuffer;
+  bool globoff;
 
   char *writeout; /* %-styled format string to output */
 
@@ -443,6 +445,46 @@ static char *file2memory(FILE *file, long *size)
     return NULL; /* no string */
 }
 
+void clean_getout(struct Configurable *config)
+{
+  struct getout *node=config->url_list;
+  struct getout *next;
+
+  while(node) {
+    next = node->next;
+    if(node->url)
+      free(node->url);
+    if(node->outfile)
+      free(node->outfile);
+    free(node);
+
+    node = next; /* GOTO next */
+  }
+}
+
+struct getout *new_getout(struct Configurable *config)
+{
+  struct getout *node =malloc(sizeof(struct getout));
+  struct getout *last= config->url_last;
+  if(node) {
+    /* clear the struct */
+    memset(node, 0, sizeof(struct getout));
+        
+    /* append this new node last in the list */
+    if(last)
+      last->next = node;
+    else
+      config->url_list = node; /* first node */
+            
+    /* move the last pointer */
+    config->url_last = node;
+  }
+  return node;
+}
+
+
+
+
 typedef enum {
   PARAM_OK,
   PARAM_OPTION_AMBIGUOUS,
@@ -470,6 +512,8 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
   time_t now;
   int hit=-1;
   bool longopt=FALSE;
+  bool singleopt=FALSE; /* when true means '-o foo' used '-ofoo' */
+
 
   /* single-letter,
      long-name,
@@ -500,7 +544,7 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
     {"Ea", "cacert",     TRUE},
     {"f", "fail",        FALSE},
     {"F", "form",        TRUE},
-
+    {"g", "globoff",     FALSE},
     {"h", "help",        FALSE},
     {"H", "header",      TRUE},
     {"i", "include",     FALSE},
@@ -600,7 +644,11 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
     if(hit < 0) {
       return PARAM_OPTION_UNKNOWN;
     }    
-    if((!nextarg || !*nextarg) && aliases[hit].extraparam) {
+    if(!longopt && aliases[hit].extraparam && parse[1]) {
+      nextarg=&parse[1]; /* this is the actual extra parameter */
+      singleopt=TRUE;   /* don't loop anymore after this */
+    }
+    else if((!nextarg || !*nextarg) && aliases[hit].extraparam) {
       return PARAM_REQUIRES_PARAMETER;
     }
     else if(nextarg && aliases[hit].extraparam)
@@ -627,7 +675,30 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
       break;
     case '5':
       /* the URL! */
-      GetStr(&config->url, nextarg);
+      {
+        struct getout *url;
+        if(config->url_get || (config->url_get=config->url_list)) {
+          /* there's a node here, if it already is filled-in continue to find
+             an "empty" node */
+          while(config->url_get && (config->url_get->flags&GETOUT_URL))
+            config->url_get = config->url_get->next;
+        }
+
+        /* now there might or might not be an available node to fill in! */
+
+        if(config->url_get)
+          /* existing node */
+          url = config->url_get;
+        else
+          /* there was no free node, create one! */
+          url=new_getout(config);
+
+        if(url) {
+          /* fill in the URL */
+          GetStr(&url->url, nextarg);
+          url->flags |= GETOUT_URL;
+        }
+      }
       break;
     case '#': /* added 19990617 larsa */
       config->progressmode ^= CURL_PROGRESS_BAR;
@@ -707,7 +778,7 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
           /* we already have a string, we append this one
              with a separating &-letter */
           char *oldpost=config->postfields;
-          config->postfields=maprintf("%s&%s", oldpost, postdata);
+          config->postfields=aprintf("%s&%s", oldpost, postdata);
           free(oldpost);
           free(postdata);
         }
@@ -766,6 +837,10 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
         return PARAM_BAD_USE;
       break;
 
+    case 'g': /* g disables URLglobbing */
+      config->globoff ^= TRUE;
+      break;
+
     case 'h': /* h for help */
       help();
       return PARAM_HELP_REQUESTED;
@@ -811,12 +886,37 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
       config->nobuffer ^= 1;
       break;
     case 'o':
-      /* output file */
-      GetStr(&config->outfile, nextarg); /* write to this file */
-      break;
     case 'O':
       /* output file */
-      config->remotefile ^= TRUE;
+      {
+        struct getout *url;
+        if(config->url_out || (config->url_out=config->url_list)) {
+          /* there's a node here, if it already is filled-in continue to find
+             an "empty" node */
+          while(config->url_out && (config->url_out->flags&GETOUT_OUTFILE))
+            config->url_out = config->url_out->next;
+        }
+
+        /* now there might or might not be an available node to fill in! */
+
+        if(config->url_out)
+          /* existing node */
+          url = config->url_out;
+        else
+          /* there was no free node, create one! */
+          url=new_getout(config);
+
+        if(url) {
+          /* fill in the outfile */
+          if('o' == letter)
+            GetStr(&url->outfile, nextarg);
+          else {
+            url->outfile=NULL; /* leave it */
+            url->flags |= GETOUT_USEREMOTE;
+          }
+          url->flags |= GETOUT_OUTFILE;
+        }
+      }
       break;
     case 'P':
       /* This makes the FTP sessions use PORT instead of PASV */
@@ -968,7 +1068,7 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
     }
     hit = -1;
 
-  } while(*++parse && !*usedarg);
+  } while(!singleopt && *++parse && !*usedarg);
 
   return PARAM_OK;
 }
@@ -1270,8 +1370,6 @@ void progressbarinit(struct ProgressData *bar)
 
 void free_config_fields(struct Configurable *config)
 {
-  if(config->url)
-    free(config->url);
   if(config->userpwd)
     free(config->userpwd);
   if(config->postfields)
@@ -1288,8 +1386,6 @@ void free_config_fields(struct Configurable *config)
     free(config->krb4level);
   if(config->headerfile)
     free(config->headerfile);
-  if(config->outfile)
-    free(config->outfile);
   if(config->ftpport)
     free(config->ftpport);
   if(config->infile)
@@ -1317,13 +1413,15 @@ operate(struct Configurable *config, int argc, char *argv[])
   char errorbuffer[CURL_ERROR_SIZE];
   char useragent[128]; /* buah, we don't want a larger default user agent */
   struct ProgressData progressbar;
+  struct getout *urlnode;
+  struct getout *nextnode;
 
   struct OutStruct outs;
   struct OutStruct heads;
 
   char *url = NULL;
 
-  URLGlob *urls;
+  URLGlob *urls=NULL;
   int urlnum;
   char *outfiles;
   int separator = 0;
@@ -1340,9 +1438,6 @@ operate(struct Configurable *config, int argc, char *argv[])
   int res;
   int i;
 
-  outs.stream = stdout;
-  outs.config = config;
-
 #ifdef MALLOCDEBUG
   /* this sends all memory debug messages to a logfile named memdump */
   curl_memdebug("memdump");
@@ -1373,7 +1468,7 @@ operate(struct Configurable *config, int argc, char *argv[])
       return res;
   }
 
-  if ((argc < 2)  && !config->url) {
+  if ((argc < 2)  && !config->url_list) {
     helpf(NULL);
     return CURLE_FAILED_INIT;
   }
@@ -1414,6 +1509,7 @@ operate(struct Configurable *config, int argc, char *argv[])
             /* no text */
             break;
           }
+          clean_getout(config);
 	  return CURLE_FAILED_INIT;
         }
 
@@ -1422,20 +1518,15 @@ operate(struct Configurable *config, int argc, char *argv[])
       }
     }
     else {
-      if(url) {
-	helpf("only one URL is supported!\n");
-	return CURLE_FAILED_INIT;
-      }
-      url = argv[i];
+      bool used;
+      /* just add the URL please */
+      res = getparameter("--url", argv[i], &used, config);
+      if(res)
+        return res;
     }
   }
 
-  /* if no URL was specified and there was one in the config file, get that
-     one */
-  if(!url && config->url)
-    url = config->url;
-  
-  if(!url) {
+  if(!config->url_list) {
     helpf("no URL specified!\n");
     return CURLE_FAILED_INIT;
   }
@@ -1447,327 +1538,355 @@ operate(struct Configurable *config, int argc, char *argv[])
   }
   else
     allocuseragent = TRUE;
-#if 0
-  fprintf(stderr, "URL: %s PROXY: %s\n", url, config->proxy?config->proxy:"none");
-#endif
 
-  /* expand '{...}' and '[...]' expressions and return total number of URLs
-     in pattern set */
-  res = glob_url(&urls, url, &urlnum);
-  if(res != CURLE_OK)
-    return res;
+  urlnode = config->url_list;
 
-  /* save outfile pattern befor expansion */
-  outfiles = config->outfile?strdup(config->outfile):NULL;
+  /* loop through the list of given URLs */
+  while(urlnode) {
 
-  if (!outfiles && !config->remotefile && urlnum > 1) {
-#ifdef CURL_SEPARATORS
-    /* multiple files extracted to stdout, insert separators! */
-    separator = 1;
-#endif
-#ifdef MIME_SEPARATORS
-    /* multiple files extracted to stdout, insert MIME separators! */
-    separator = 1;
-    printf("MIME-Version: 1.0\n");
-    printf("Content-Type: multipart/mixed; boundary=%s\n\n", MIMEseparator);
-#endif
-  }
-  for (i = 0; (url = next_url(urls)); ++i) {
-    if (outfiles) {
-      free(config->outfile);
-      config->outfile = outfiles;
+    /* get the full URL (it might be NULL) */
+    url=urlnode->url;
+
+    if(NULL == url) {
+      /* This node had no URL, skip it and continue to the next */
+      if(urlnode->outfile)
+        free(urlnode->outfile);
+    
+      /* move on to the next URL */
+      nextnode=urlnode->next;
+      free(urlnode); /* free the node */
+      urlnode = nextnode;
+      continue; /* next please */
     }
+
+    /* default output stream is stdout */
+    outs.stream = stdout;
+    outs.config = config;
+
+    if(!config->globoff) {
+      /* Unless explicitly shut off, we expand '{...}' and '[...]' expressions
+         and return total number of URLs in pattern set */
+      res = glob_url(&urls, url, &urlnum);
+      if(res != CURLE_OK)
+        return res;
+    }
+
+
+    /* save outfile pattern before expansion */
+    outfiles = urlnode->outfile?strdup(urlnode->outfile):NULL;
+
+    if ((!outfiles || strequal(outfiles, "-")) && urlnum > 1) {
+      /* multiple files extracted to stdout, insert separators! */
+      separator = 1;
+    }
+    for(i = 0;
+        (url = urls?next_url(urls):(i?NULL:strdup(url)));
+        i++) {
+      char *outfile;
+      outfile = outfiles?strdup(outfiles):NULL;
  
-    if (config->outfile || config->remotefile) {
-      /* 
-       * We have specified a file name to store the result in, or we have
-       * decided we want to use the remote file name.
-       */
+      if((urlnode->flags&GETOUT_USEREMOTE) ||
+         (outfile && !strequal("-", outfile)) ) {
+
+        /* 
+         * We have specified a file name to store the result in, or we have
+         * decided we want to use the remote file name.
+         */
       
-      if(!config->outfile && config->remotefile) {
-        /* Find and get the remote file name */
-        config->outfile=strstr(url, "://");
-        if(config->outfile)
-          config->outfile+=3;
-        else
-          config->outfile=url;
-        config->outfile = strdup(strrchr(config->outfile, '/'));
-        if(!config->outfile || !strlen(++config->outfile)) {
-          helpf("Remote file name has no length!\n");
-          return CURLE_WRITE_ERROR;
+        if(!outfile) {
+          /* Find and get the remote file name */
+          char * pc =strstr(url, "://");
+          if(pc)
+            pc+=3;
+          else
+            pc=url;
+          pc = strrchr(pc, '/');
+          outfile = (char *) NULL == pc ? NULL : strdup(pc+1) ;
+          if(!outfile) {
+            helpf("Remote file name has no length!\n");
+            return CURLE_WRITE_ERROR;
+          }
+        }
+        else if(urls) {
+          /* fill '#1' ... '#9' terms from URL pattern */
+          char *storefile = outfile;
+          outfile = match_url(storefile, urls);
+          free(storefile);
+        }
+      
+        if((0 == config->resume_from) && config->use_resume) {
+          /* we're told to continue where we are now, then we get the size of
+             the file as it is now and open it for append instead */
+
+          struct stat fileinfo;
+
+          if(0 == stat(outfile, &fileinfo)) {
+            /* set offset to current file size: */
+            config->resume_from = fileinfo.st_size;
+          }
+          /* else let offset remain 0 */
+        }
+      
+        if(config->resume_from) {
+          /* open file for output: */
+          outs.stream=(FILE *) fopen(outfile, config->resume_from?"ab":"wb");
+          if (!outs.stream) {
+            helpf("Can't open '%s'!\n", outfile);
+            return CURLE_WRITE_ERROR;
+          }
+        }
+        else {
+          outs.filename = outfile;
+          outs.stream = NULL; /* open when needed */
         }
       }
-      else {
-	/* fill '#1' ... '#9' terms from URL pattern */
-        char *outfile = config->outfile;
-        config->outfile = match_url(config->outfile, *urls);
-        free(outfile);
-      }
-      
-      if((0 == config->resume_from) && config->use_resume) {
-        /* we're told to continue where we are now, then we get the size of the
-           file as it is now and open it for append instead */
+      if(config->infile) {
+        /*
+         * We have specified a file to upload
+         */
         struct stat fileinfo;
 
-        if(0 == stat(config->outfile, &fileinfo)) {
-          /* set offset to current file size: */
-          config->resume_from = fileinfo.st_size;
-        }
-        /* else let offset remain 0 */
-      }
-      
-      if(config->resume_from) {
-        /* open file for output: */
-        outs.stream=(FILE *) fopen(config->outfile, config->resume_from?"ab":"wb");
-        if (!outs.stream) {
-          helpf("Can't open '%s'!\n", config->outfile);
-          return CURLE_WRITE_ERROR;
-        }
-      }
-      else {
-        outs.filename = config->outfile;
-        outs.stream = NULL; /* open when needed */
-      }
-    }
-    if (config->infile) {
-      /*
-       * We have specified a file to upload
-       */
-      struct stat fileinfo;
-
-      /* If no file name part is given in the URL, we add this file name */
-      char *ptr=strstr(url, "://");
-      if(ptr)
-        ptr+=3;
-      else
-        ptr=url;
-      ptr = strrchr(ptr, '/');
-      if(!ptr || !strlen(++ptr)) {
-        /* The URL has no file name part, add the local file name. In order to
-           be able to do so, we have to create a new URL in another buffer.*/
-
-        urlbuffer=(char *)malloc(strlen(url) + strlen(config->infile) + 3);
-        if(!urlbuffer) {
-          helpf("out of memory\n");
-          return CURLE_OUT_OF_MEMORY;
-        }
+        /* If no file name part is given in the URL, we add this file name */
+        char *ptr=strstr(url, "://");
         if(ptr)
-          /* there is a trailing slash on the URL */
-          sprintf(urlbuffer, "%s%s", url, config->infile);
+          ptr+=3;
         else
-          /* thers is no trailing slash on the URL */
-          sprintf(urlbuffer, "%s/%s", url, config->infile);
-        
-        url = urlbuffer; /* use our new URL instead! */
+          ptr=url;
+        ptr = strrchr(ptr, '/');
+        if(!ptr || !strlen(++ptr)) {
+          /* The URL has no file name part, add the local file name. In order
+             to be able to do so, we have to create a new URL in another
+             buffer.*/
+
+          urlbuffer=(char *)malloc(strlen(url) + strlen(config->infile) + 3);
+          if(!urlbuffer) {
+            helpf("out of memory\n");
+            return CURLE_OUT_OF_MEMORY;
+          }
+          if(ptr)
+            /* there is a trailing slash on the URL */
+            sprintf(urlbuffer, "%s%s", url, config->infile);
+          else
+            /* thers is no trailing slash on the URL */
+            sprintf(urlbuffer, "%s/%s", url, config->infile);
+          
+          url = urlbuffer; /* use our new URL instead! */
+        }
+
+        infd=(FILE *) fopen(config->infile, "rb");
+        if (!infd || stat(config->infile, &fileinfo)) {
+          helpf("Can't open '%s'!\n", config->infile);
+          return CURLE_READ_ERROR;
+        }
+        infilesize=fileinfo.st_size;
+      
+      }
+      if((config->conf&CONF_UPLOAD) &&
+         config->use_resume &&
+         (0==config->resume_from)) {
+        config->resume_from = -1; /* -1 will then force get-it-yourself */
+      }
+      if(config->headerfile) {
+        /* open file for output: */
+        if(strcmp(config->headerfile,"-")) {
+          heads.filename = config->headerfile;
+          headerfilep=NULL;
+        }
+        else
+          headerfilep=stdout;
+        heads.stream = headerfilep;
+        heads.config = config;
+      }
+    
+      if(outs.stream && isatty(fileno(outs.stream)) &&
+         !(config->conf&(CONF_UPLOAD|CONF_HTTPPOST)))
+        /* we send the output to a tty and it isn't an upload operation,
+           therefore we switch off the progress meter */
+        config->conf |= CONF_NOPROGRESS;
+    
+
+      if (urlnum > 1) {
+        fprintf(stderr, "\n[%d/%d]: %s --> %s\n",
+                i+1, urlnum, url, outfile ? outfile : "<stdout>");
+        if (separator)
+          printf("%s%s\n", CURLseparator, url);
       }
 
-      infd=(FILE *) fopen(config->infile, "rb");
-      if (!infd || stat(config->infile, &fileinfo)) {
-        helpf("Can't open '%s'!\n", config->infile);
-        return CURLE_READ_ERROR;
+      if(!config->errors)
+        config->errors = stderr;
+
+#if defined(WIN32) && !defined(__CYGWIN32__)
+      if(!outfile && !(config->conf & CONF_GETTEXT)) {
+        /* We get the output to stdout and we have not got the ASCII/text flag,
+           then set stdout to be binary */
+        setmode( 1, O_BINARY );
       }
-      infilesize=fileinfo.st_size;
+#endif
+
+
+      main_init();
+
+      curl = curl_easy_init();
+      if(curl) {
+        curl_easy_setopt(curl, CURLOPT_FILE, (FILE *)&outs); /* where to store */
+        /* what call to write: */
+        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, my_fwrite);
+        curl_easy_setopt(curl, CURLOPT_INFILE, infd); /* for uploads */
+        /* size of uploaded file: */
+        curl_easy_setopt(curl, CURLOPT_INFILESIZE, infilesize);
+        curl_easy_setopt(curl, CURLOPT_URL, url);     /* what to fetch */
+        curl_easy_setopt(curl, CURLOPT_PROXY, config->proxy); /* proxy to use */
+        curl_easy_setopt(curl, CURLOPT_VERBOSE, config->conf&CONF_VERBOSE);
+        curl_easy_setopt(curl, CURLOPT_HEADER, config->conf&CONF_HEADER);
+        curl_easy_setopt(curl, CURLOPT_NOPROGRESS, config->conf&CONF_NOPROGRESS);
+        curl_easy_setopt(curl, CURLOPT_NOBODY, config->conf&CONF_NOBODY);
+        curl_easy_setopt(curl, CURLOPT_FAILONERROR,
+                         config->conf&CONF_FAILONERROR);
+        curl_easy_setopt(curl, CURLOPT_UPLOAD, config->conf&CONF_UPLOAD);
+        curl_easy_setopt(curl, CURLOPT_POST, config->conf&CONF_POST);
+        curl_easy_setopt(curl, CURLOPT_FTPLISTONLY,
+                         config->conf&CONF_FTPLISTONLY);
+        curl_easy_setopt(curl, CURLOPT_FTPAPPEND, config->conf&CONF_FTPAPPEND);
+        curl_easy_setopt(curl, CURLOPT_NETRC, config->conf&CONF_NETRC);
+        curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION,
+                         config->conf&CONF_FOLLOWLOCATION);
+        curl_easy_setopt(curl, CURLOPT_TRANSFERTEXT, config->conf&CONF_GETTEXT);
+#if 0
+        curl_easy_setopt(curl, CURLOPT_PUT, config->conf&CONF_PUT);
+#endif
+        curl_easy_setopt(curl, CURLOPT_MUTE, config->conf&CONF_MUTE);
+        curl_easy_setopt(curl, CURLOPT_USERPWD, config->userpwd);
+        curl_easy_setopt(curl, CURLOPT_PROXYUSERPWD, config->proxyuserpwd);
+        curl_easy_setopt(curl, CURLOPT_RANGE, config->range);
+        curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errorbuffer);
+        curl_easy_setopt(curl, CURLOPT_TIMEOUT, config->timeout);
+        curl_easy_setopt(curl, CURLOPT_POSTFIELDS, config->postfields);
+        
+        /* new in libcurl 7.2: */
+        curl_easy_setopt(curl, CURLOPT_POSTFIELDSIZE, config->postfieldsize);
+        
+        curl_easy_setopt(curl, CURLOPT_REFERER, config->referer);
+        curl_easy_setopt(curl, CURLOPT_AUTOREFERER,
+                         config->conf&CONF_AUTO_REFERER);
+        curl_easy_setopt(curl, CURLOPT_USERAGENT, config->useragent);
+        curl_easy_setopt(curl, CURLOPT_FTPPORT, config->ftpport);
+        curl_easy_setopt(curl, CURLOPT_LOW_SPEED_LIMIT, config->low_speed_limit);
+        curl_easy_setopt(curl, CURLOPT_LOW_SPEED_TIME, config->low_speed_time);
+        curl_easy_setopt(curl, CURLOPT_RESUME_FROM,
+                         config->use_resume?config->resume_from:0);
+        curl_easy_setopt(curl, CURLOPT_COOKIE, config->cookie);
+        curl_easy_setopt(curl, CURLOPT_HTTPHEADER, config->headers);
+        curl_easy_setopt(curl, CURLOPT_HTTPPOST, config->httppost);
+        curl_easy_setopt(curl, CURLOPT_SSLCERT, config->cert);
+        curl_easy_setopt(curl, CURLOPT_SSLCERTPASSWD, config->cert_passwd);
+
+        if(config->cacert) {
+          /* available from libcurl 7.5: */
+          curl_easy_setopt(curl, CURLOPT_CAINFO, config->cacert);
+          curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, TRUE);
+        }
+
+        if(config->conf&(CONF_NOBODY|CONF_USEREMOTETIME)) {
+          /* no body or use remote time */
+          /* new in 7.5 */
+          curl_easy_setopt(curl, CURLOPT_FILETIME, TRUE);
+        }
       
-    }
-    if((config->conf&CONF_UPLOAD) &&
-       config->use_resume &&
-       (0==config->resume_from)) {
-      config->resume_from = -1; /* -1 will then force get-it-yourself */
-    }
-    if(config->headerfile) {
-      /* open file for output: */
-      if(strcmp(config->headerfile,"-")) {
-        heads.filename = config->headerfile;
-        headerfilep=NULL;
+        /* 7.5 news: */
+        if (config->maxredirs) 
+          curl_easy_setopt(curl, CURLOPT_MAXREDIRS, config->maxredirs); 
+        else 
+          curl_easy_setopt(curl, CURLOPT_MAXREDIRS, DEFAULT_MAXREDIRS); 
+ 
+        curl_easy_setopt(curl, CURLOPT_CRLF, config->crlf);
+        curl_easy_setopt(curl, CURLOPT_QUOTE, config->quote);
+        curl_easy_setopt(curl, CURLOPT_POSTQUOTE, config->postquote);
+        curl_easy_setopt(curl, CURLOPT_WRITEHEADER,
+                         config->headerfile?&heads:NULL);
+        curl_easy_setopt(curl, CURLOPT_COOKIEFILE, config->cookiefile);
+        curl_easy_setopt(curl, CURLOPT_SSLVERSION, config->ssl_version);
+        curl_easy_setopt(curl, CURLOPT_TIMECONDITION, config->timecond);
+        curl_easy_setopt(curl, CURLOPT_TIMEVALUE, config->condtime);
+        curl_easy_setopt(curl, CURLOPT_CUSTOMREQUEST, config->customrequest);
+        curl_easy_setopt(curl, CURLOPT_STDERR, config->errors);
+      
+        /* three new ones in libcurl 7.3: */
+        curl_easy_setopt(curl, CURLOPT_HTTPPROXYTUNNEL, config->proxytunnel);
+        curl_easy_setopt(curl, CURLOPT_INTERFACE, config->iface);
+        curl_easy_setopt(curl, CURLOPT_KRB4LEVEL, config->krb4level);
+
+        if((config->progressmode == CURL_PROGRESS_BAR) &&
+           !(config->conf&(CONF_NOPROGRESS|CONF_MUTE))) {
+          /* we want the alternative style, then we have to implement it
+             ourselves! */
+          progressbarinit(&progressbar);
+          curl_easy_setopt(curl, CURLOPT_PROGRESSFUNCTION, myprogress);
+          curl_easy_setopt(curl, CURLOPT_PROGRESSDATA, &progressbar);
+        }
+        
+        res = curl_easy_perform(curl);
+        
+        if(config->writeout) {
+          ourWriteOut(curl, config->writeout);
+        }
+        
+        /* always cleanup */
+        curl_easy_cleanup(curl);
+        
+        if((res!=CURLE_OK) && config->showerror)
+          fprintf(config->errors, "curl: (%d) %s\n", res, errorbuffer);
       }
       else
-        headerfilep=stdout;
-      heads.stream = headerfilep;
-      heads.config = config;
-    }
-    
-    if(outs.stream && isatty(fileno(outs.stream)) &&
-       !(config->conf&(CONF_UPLOAD|CONF_HTTPPOST)))
-      /* we send the output to a tty and it isn't an upload operation,
-         therefore we switch off the progress meter */
-      config->conf |= CONF_NOPROGRESS;
+        fprintf(config->errors, "curl: failed to init libcurl!\n");
+
+      main_free();
+
+      if((config->errors != stderr) &&
+         (config->errors != stdout))
+        /* it wasn't directed to stdout or stderr so close the file! */
+        fclose(config->errors);
     
+      if(config->headerfile && !headerfilep && heads.stream)
+        fclose(heads.stream);
 
-    if (urlnum > 1) {
-      fprintf(stderr, "\n[%d/%d]: %s --> %s\n",
-              i+1, urlnum, url, config->outfile ? config->outfile : "<stdout>");
-      if (separator) {
-#ifdef CURL_SEPARATORS
-        printf("%s%s\n", CURLseparator, url);
-#endif
-#ifdef MIME_SEPARATORS
-        printf("--%s\n", MIMEseparator);
-        printf("Content-ID: %s\n\n", url); 
-#endif
-      }
-    }
-
-    if(!config->errors)
-      config->errors = stderr;
-
-#ifdef WIN32
-    if(!config->outfile && !(config->conf & CONF_GETTEXT)) {
-      /* We get the output to stdout and we have not got the ASCII/text flag,
-         then set stdout to be binary */
-      setmode( 1, O_BINARY );
-    }
-#endif
-
-
-    main_init();
-
-    /* The new, v7-style easy-interface! */
-    curl = curl_easy_init();
-    if(curl) {
-      curl_easy_setopt(curl, CURLOPT_FILE, (FILE *)&outs); /* where to store */
-      /* what call to write: */
-      curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, my_fwrite);
-      curl_easy_setopt(curl, CURLOPT_INFILE, infd); /* for uploads */
-      /* size of uploaded file: */
-      curl_easy_setopt(curl, CURLOPT_INFILESIZE, infilesize);
-      curl_easy_setopt(curl, CURLOPT_URL, url);     /* what to fetch */
-      curl_easy_setopt(curl, CURLOPT_PROXY, config->proxy); /* proxy to use */
-      curl_easy_setopt(curl, CURLOPT_VERBOSE, config->conf&CONF_VERBOSE);
-      curl_easy_setopt(curl, CURLOPT_HEADER, config->conf&CONF_HEADER);
-      curl_easy_setopt(curl, CURLOPT_NOPROGRESS, config->conf&CONF_NOPROGRESS);
-      curl_easy_setopt(curl, CURLOPT_NOBODY, config->conf&CONF_NOBODY);
-      curl_easy_setopt(curl, CURLOPT_FAILONERROR,
-                       config->conf&CONF_FAILONERROR);
-      curl_easy_setopt(curl, CURLOPT_UPLOAD, config->conf&CONF_UPLOAD);
-      curl_easy_setopt(curl, CURLOPT_POST, config->conf&CONF_POST);
-      curl_easy_setopt(curl, CURLOPT_FTPLISTONLY,
-                       config->conf&CONF_FTPLISTONLY);
-      curl_easy_setopt(curl, CURLOPT_FTPAPPEND, config->conf&CONF_FTPAPPEND);
-      curl_easy_setopt(curl, CURLOPT_NETRC, config->conf&CONF_NETRC);
-      curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION,
-                       config->conf&CONF_FOLLOWLOCATION);
-      curl_easy_setopt(curl, CURLOPT_TRANSFERTEXT, config->conf&CONF_GETTEXT);
-      curl_easy_setopt(curl, CURLOPT_PUT, config->conf&CONF_PUT);
-      curl_easy_setopt(curl, CURLOPT_MUTE, config->conf&CONF_MUTE);
-      curl_easy_setopt(curl, CURLOPT_USERPWD, config->userpwd);
-      curl_easy_setopt(curl, CURLOPT_PROXYUSERPWD, config->proxyuserpwd);
-      curl_easy_setopt(curl, CURLOPT_RANGE, config->range);
-      curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errorbuffer);
-      curl_easy_setopt(curl, CURLOPT_TIMEOUT, config->timeout);
-      curl_easy_setopt(curl, CURLOPT_POSTFIELDS, config->postfields);
+      if(urlbuffer)
+        free(urlbuffer);
+      if (outfile && !strequal(outfile, "-") && outs.stream)
+        fclose(outs.stream);
+      if (config->infile)
+        fclose(infd);
+      if(headerfilep)
+        fclose(headerfilep);
       
-      /* new in libcurl 7.2: */
-      curl_easy_setopt(curl, CURLOPT_POSTFIELDSIZE, config->postfieldsize);
+      if(url)
+        free(url);
 
-      curl_easy_setopt(curl, CURLOPT_REFERER, config->referer);
-      curl_easy_setopt(curl, CURLOPT_AUTOREFERER,
-                       config->conf&CONF_AUTO_REFERER);
-      curl_easy_setopt(curl, CURLOPT_USERAGENT, config->useragent);
-      curl_easy_setopt(curl, CURLOPT_FTPPORT, config->ftpport);
-      curl_easy_setopt(curl, CURLOPT_LOW_SPEED_LIMIT, config->low_speed_limit);
-      curl_easy_setopt(curl, CURLOPT_LOW_SPEED_TIME, config->low_speed_time);
-      curl_easy_setopt(curl, CURLOPT_RESUME_FROM,
-                       config->use_resume?config->resume_from:0);
-      curl_easy_setopt(curl, CURLOPT_COOKIE, config->cookie);
-      curl_easy_setopt(curl, CURLOPT_HTTPHEADER, config->headers);
-      curl_easy_setopt(curl, CURLOPT_HTTPPOST, config->httppost);
-      curl_easy_setopt(curl, CURLOPT_SSLCERT, config->cert);
-      curl_easy_setopt(curl, CURLOPT_SSLCERTPASSWD, config->cert_passwd);
-
-      if(config->cacert) {
-        /* available from libcurl 7.5: */
-        curl_easy_setopt(curl, CURLOPT_CAINFO, config->cacert);
-        curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, TRUE);
-      }
-
-      if(config->conf&(CONF_NOBODY|CONF_USEREMOTETIME)) {
-        /* no body or use remote time */
-        /* new in 7.5 */
-        curl_easy_setopt(curl, CURLOPT_FILETIME, TRUE);
-      }
-      
-      /* 7.5 news: */
-      if (config->maxredirs) 
-        curl_easy_setopt(curl, CURLOPT_MAXREDIRS, config->maxredirs); 
-      else 
-        curl_easy_setopt(curl, CURLOPT_MAXREDIRS, DEFAULT_MAXREDIRS); 
- 
-
-      curl_easy_setopt(curl, CURLOPT_CRLF, config->crlf);
-      curl_easy_setopt(curl, CURLOPT_QUOTE, config->quote);
-      curl_easy_setopt(curl, CURLOPT_POSTQUOTE, config->postquote);
-      curl_easy_setopt(curl, CURLOPT_WRITEHEADER,
-                       config->headerfile?&heads:NULL);
-      curl_easy_setopt(curl, CURLOPT_COOKIEFILE, config->cookiefile);
-      curl_easy_setopt(curl, CURLOPT_SSLVERSION, config->ssl_version);
-      curl_easy_setopt(curl, CURLOPT_TIMECONDITION, config->timecond);
-      curl_easy_setopt(curl, CURLOPT_TIMEVALUE, config->condtime);
-      curl_easy_setopt(curl, CURLOPT_CUSTOMREQUEST, config->customrequest);
-      curl_easy_setopt(curl, CURLOPT_STDERR, config->errors);
-      
-      /* three new ones in libcurl 7.3: */
-      curl_easy_setopt(curl, CURLOPT_HTTPPROXYTUNNEL, config->proxytunnel);
-      curl_easy_setopt(curl, CURLOPT_INTERFACE, config->iface);
-      curl_easy_setopt(curl, CURLOPT_KRB4LEVEL, config->krb4level);
-
-      if((config->progressmode == CURL_PROGRESS_BAR) &&
-         !(config->conf&(CONF_NOPROGRESS|CONF_MUTE))) {
-        /* we want the alternative style, then we have to implement it
-           ourselves! */
-        progressbarinit(&progressbar);
-        curl_easy_setopt(curl, CURLOPT_PROGRESSFUNCTION, myprogress);
-        curl_easy_setopt(curl, CURLOPT_PROGRESSDATA, &progressbar);
-      }
-
-      res = curl_easy_perform(curl);
-
-      if(config->writeout) {
-        ourWriteOut(curl, config->writeout);
-      }
-
-      /* always cleanup */
-      curl_easy_cleanup(curl);
-
-      if((res!=CURLE_OK) && config->showerror)
-        fprintf(config->errors, "curl: (%d) %s\n", res, errorbuffer);
+      if(outfile)
+        free(outfile);
     }
-    else
-      fprintf(config->errors, "curl: failed to init libcurl!\n");
+    if(outfiles)
+      free(outfiles);
 
-    main_free();
+    if(urls)
+      /* cleanup memory used for URL globbing patterns */
+      glob_cleanup(urls);
 
-    if((config->errors != stderr) &&
-       (config->errors != stdout))
-      /* it wasn't directed to stdout or stderr so close the file! */
-      fclose(config->errors);
+    /* empty this urlnode struct */
+    if(urlnode->url)
+      free(urlnode->url);
+    if(urlnode->outfile)
+      free(urlnode->outfile);
     
-    if(config->headerfile && !headerfilep && heads.stream)
-      fclose(heads.stream);
+    /* move on to the next URL */
+    nextnode=urlnode->next;
+    free(urlnode); /* free the node */
+    urlnode = nextnode;
 
-    if(urlbuffer)
-      free(urlbuffer);
-    if (config->outfile && outs.stream)
-      fclose(outs.stream);
-    if (config->infile)
-      fclose(infd);
-    if(headerfilep)
-      fclose(headerfilep);
-    
-    if(url)
-      free(url);
-
-  }
-#ifdef MIME_SEPARATORS
-  if (separator)
-    printf("--%s--\n", MIMEseparator);
-#endif
+  } /* while-loop through all URLs */
 
   if(allocuseragent)
     free(config->useragent);
 
-  /* cleanup memory used for URL globbing patterns */
-  glob_cleanup(urls);
-
   return res;
 }
 
